/*      file     : Planner/synchronize.cc
        project  : PAMINA III
        author   : Marcus Rickert
        RCS      : $Id: synchronize.cc,v 1.4 2005/03/25 12:27:31 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// $Id: synchronize.cc,v 1.4 2005/03/25 12:27:31 mr Exp $

#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>

#include "synchronize.h"

// =================================================================================== 

int
SyncActivate(TSyncInfo *Info, const char *thePipeName, int IsMasterFlag)

{
    char ReadFilename[128];
    char WriteFilename[128];

    strcpy(Info->PipeName, thePipeName);
    Info->IsMaster = IsMasterFlag;

    if (Info->IsMaster)
    {
	sprintf(WriteFilename, "%s.master", Info->PipeName);
	sprintf(ReadFilename, "%s.slave", Info->PipeName);
    }
    else
    {
	sprintf(ReadFilename, "%s.master", Info->PipeName);
	sprintf(WriteFilename, "%s.slave", Info->PipeName);
    }
    if ((Info->Read = open(ReadFilename, O_RDONLY | O_NONBLOCK)) == -1)
    {
	fprintf(stderr, "activate: cannot open pipe %s for reading\n", ReadFilename);
	Info->Read = 0;
	return 0;
    }
    if ((Info->Write = open(WriteFilename, O_WRONLY)) == -1)
    {
	fprintf(stderr, "Activate: cannot open pipe %s for writing\n", WriteFilename);
	Info->Write = 0;
	return 0;
    }
    return 1;
}

// =================================================================================== 

void
SyncDeactivate(TSyncInfo *Info)

{
    if (Info->Write)
    {
	close(Info->Write);
	Info->Write = 0;
    }
    sleep(1);
    if (Info->Read)
    {
	close(Info->Read);
	Info->Read = 0;
    }
}

// =================================================================================== 

int
SyncStep(TSyncInfo *Info, long Step)

{
    long NrOfBytes;
    
    if (Info->Write == 0)
    {
	fprintf(stderr, "SyncStep: not active\n");
	return 0;
    }
    if ((NrOfBytes = write(Info->Write, &Step, sizeof(long))) != sizeof(long))
    {
	fprintf(stderr, "SyncStep: cannot write to pipe\n");
	return 0;
    }
    return 1;   
}

// =================================================================================== 

void
SyncInit(TSyncInfo *Info)

{
    Info->Read = 0;
    Info->Write = 0;
}

// =================================================================================== 

int
SyncWait(TSyncInfo *Info, long &MyStep, int &ErrorFlag)

{
    long Step;
    long NrOfBytes;

    ErrorFlag = 0;
    if (Info->Read == 0)
    {
	fprintf(stderr, "SyncWait: not active\n");
	ErrorFlag = 1;
	return 0;
    }
    errno = 0;
    if ((NrOfBytes = read(Info->Read, &Step, sizeof(long))) != sizeof(long))
    {
	if (errno == EAGAIN)
	    return 1;
	else
	{
	    ErrorFlag = 1;
	    return 0;
	}
    }
    if (Step >= MyStep)
    {
	MyStep = Step;
	return 0;
    }
    return 1;
}

// =================================================================================== 
