/*      file     : TrafficData/CAConfig.h
        project  : multi lane CA
        author   : Marcus Rickert
        RCS      : $Id: CAConfig.h,v 1.11 2005/03/26 06:58:51 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef CACONFIG_H
#define CACONFIG_H

// ***INCLUDES************************************************************************

#include <limits.h>

// ***DEFINES*************************************************************************

#define CA_GROUP                        "CA"

#define CA_NAME_INITDENSITY             "InitDensity"
#define CA_NAME_STARTDUMP               "StartDump"
#define CA_NAME_STOPDUMP                "StopDump"
#define CA_NAME_DUMPWIDTH               "DumpWidth"
#define CA_NAME_PASSINGPROHIBITION      "PassingProhibition"
#define CA_NAME_SPEEDLIMITS             "SpeedLimits"
#define CA_NAME_MINSITES                "MinSites"
#define CA_NAME_DECPROB                 "DecProb"

#define CA_DEFAULT_V_MAX                5

#define CA_DEFAULT_INITDENSITY          0.0
#define CA_DEFAULT_STARTDUMP            -1L
#define CA_DEFAULT_STOPDUMP             -1L
#define CA_DEFAULT_DUMPWIDTH            50L
#define CA_DEFAULT_PASSINGPROHIBITION   0L
#define CA_DEFAULT_MINSITES             ((long) CA_DEFAULT_V_MAX)
#define CA_DEFAULT_SPEEDLIMITS          0L

#define MAX_LANES                       6
#define MAX_BOUNDARY_WIDTH              (4 * CA_DEFAULT_V_MAX)
#define MAX_SITES_PER_GRID              2000

#define CA_ASYMMETRIC                   0x0001
#define CA_PASSING_PROHIBITION          0x0002
#define CA_BLOCK_SOURCE                 0x0004
#define CA_BLOCK_DEST                   0x0008
#define CA_SUPPRESS_P_SOURCE            0x0010   // suppress passing at source
#define CA_SUPPRESS_P_DEST              0x0020   // suppress passing at destination
#define CA_SKIP_WHEN_EMPTY              0x0040
#define CA_FLOOR_GRID_LENGTH            0x0080
#define CA_DEFAULT_FLAGS                (CA_ASYMMETRIC | CA_SKIP_WHEN_EMPTY)

#define CA_DEFAULT_REDUCED_GAP          3
#define CA_DEFAULT_SPEED_LIMIT          CA_DEFAULT_V_MAX
#define CA_DEFAULT_NR_OF_LANES          0
#define CA_DEFAULT_MAX_VELOCITY         CA_DEFAULT_V_MAX
#define CA_DEFAULT_MOVE_UPDATE_STEPS    1
#define CA_DEFAULT_MOVE_VIEW_RANGE      CA_DEFAULT_V_MAX
#define CA_DEFAULT_CHANGE_UPDATE_STEPS  1
#define CA_DEFAULT_CHANGE_VIEW_RANGE    CA_DEFAULT_V_MAX
#define CA_DEFAULT_DEC_PROB             0.5
#define CA_DEFAULT_LEFT_LOOKBACK        CA_DEFAULT_V_MAX
#define CA_DEFAULT_RIGHT_LOOKBACK       CA_DEFAULT_V_MAX
#define CA_DEFAULT_ROUTE_LEFT_LOOKBACK  0
#define CA_DEFAULT_ROUTE_RIGHT_LOOKBACK 0

#define CA_DEFAULT_LR_PROB             0.5   
#define CA_DEFAULT_RL_PROB             0.5

#define SITE_V_DEFAULT              0
#define SITE_DEFAULT_V_MAX          5
#define FAST_PING_PONG_V_MIN        4

#define CA_TO_REAL_FACTOR 24.0
#define CA_VELOCITY_TO_REAL_VELOCITY(CAV) (CA_TO_REAL_FACTOR * (double) (CAV))
#define REAL_VELOCITY_TO_CA_VELOCITY(REALV) ((long) ((REALV) / CA_TO_REAL_FACTOR + 0.5))
#define SITE_LENGTH 7.5
#define SITE_LENGTH_IN_CM 750.0
#define CM_TO_SITES(CM) ((CM) / SITE_LENGTH_IN_CM)

#define CHANGE_LEFT     0
#define CHANGE_RIGHT    1
#define LEFT_LANE       0  // for two lanes
#define RIGHT_LANE      1  // for two lanes
#define ALL_LANES       (MAX_LANES)
#define ALL_V           (CA_DEFAULT_V_MAX + 1)
#define BOTH_DIRECTIONS (CHANGE_RIGHT + 1)

// bit configuration (simple characteristics) of a TSite

#define SITE_VELOCITY_MASK             7
#define SITE_V_MAX_SHIFT               3
#define SITE_V_MAX_MASK               (7 << SITE_V_MAX_SHIFT)      
#define SITE_V_OLD_SHIFT               6
#define SITE_V_OLD_MASK               (7 << SITE_V_OLD_SHIFT)
#define SITE_V_OLD2_SHIFT              9
#define SITE_V_OLD2_MASK              (7 << SITE_V_OLD2_SHIFT)

#define SITE_DELAY_MASK              SITE_V_OLD2_MASK
#define SITE_DELAY_SHIFT             SITE_V_OLD2_SHIFT

#define SITE_DEC_PROB_RANDOM_FLAG    0x00001000
#define SITE_VEHICLE                 0x00002000
#define SITE_TMP_NO_CHANGE_LEFT      0x00004000
#define SITE_BLOCKED                 0x00008000

#define SITE_HIGHLIGHT               0x00010000
#define SITE_PREV_TS_LANE_CHANGE     0x00020000
#define SITE_ABSORPTION              0x00040000
#define SITE_EMISSION                0x00080000

#define SITE_PERM_NO_CHANGE_LEFT     0x00100000
#define SITE_PERM_NO_CHANGE_RIGHT    0x00200000

// MOVE_LEFT and MOVE_RIGHT are used for forced lane changing by route plans

#define SITE_MOVE_LEFT               0x00400000
#define SITE_MOVE_RIGHT              0x00800000

#define SITE_BND_LANE_SHIFT          24
#define SITE_BND_LANE_MASK           (7 << SITE_BND_LANE_SHIFT)

#define SITE_MARK_EXIT               (1 << 27)
#define SITE_MARK_MOVE_LEFT          (SITE_MARK_EXIT)
#define SITE_MARK_MOVE_RIGHT         (SITE_MARK_EXIT)
#define SITE_CHECK_BLOCK             (1 << 28)
#define SITE_ROUTE_BLOCK             (1 << 29)
#define SITE_REROUTED_VEHICLE        (1 << 30)

// combined characteristics

#define SITE_NO_CHANGE_LEFT           (SITE_TMP_NO_CHANGE_LEFT | SITE_PERM_NO_CHANGE_LEFT)
#define SITE_NO_CHANGE_RIGHT          (SITE_PERM_NO_CHANGE_RIGHT)
#define SITE_NO_CHANGE                (SITE_NO_CHANGE_LEFT | SITE_NO_CHANGE_RIGHT)
#define SITE_PERM_NO_CHANGE           (SITE_PERM_NO_CHANGE_LEFT | SITE_PERM_NO_CHANGE_RIGHT)

#define SITE_EXTRA               (SITE_BLOCKED | SITE_ABSORPTION | SITE_EMISSION | \
				  SITE_PERM_NO_CHANGE | SITE_CHECK_BLOCK | \
				  SITE_ROUTE_BLOCK)

//#define SITE_EXTRA               (SITE_BLOCKED | SITE_ABSORPTION | SITE_EMISSION | 
//				  SITE_PERM_NO_CHANGE | SITE_MARK_MOVE_LEFT | 
//				  SITE_MARK_MOVE_RIGHT | SITE_CHECK_BLOCK | 
//				  SITE_ROUTE_BLOCK)

#define SITE_SPECIAL             (SITE_ABSORPTION | SITE_EMISSION | \
                                  SITE_TRANSFER_ORIGIN | SITE_TRANSFER_DEST | \
				  SITE_CHECK_BLOCK | SITE_ROUTE_BLOCK)

// The SITE_MARK_FLAGS flags must be deleted after every successful transfer to a 
// a new segment.

#define SITE_MARK_FLAGS         (SITE_MOVE_LEFT | SITE_MOVE_RIGHT | SITE_BND_LANE_MASK)

// constant vehicle date with respect to the single lane update; note that the
// flags for the lane changing probabilities are included!

#define SITE_CONST_VEHICLE_MASK (SITE_VEHICLE | SITE_V_MAX_MASK|\
				 SITE_HIGHLIGHT | SITE_PREV_TS_LANE_CHANGE |\
				 SITE_MARK_FLAGS | SITE_REROUTED_VEHICLE)

// variable vehicle data with respect to the single lane update

#define SITE_VAR_VEHICLE_MASK (SITE_VELOCITY_MASK | \
			       SITE_V_OLD_MASK | SITE_V_OLD2_MASK | \
			       SITE_DEC_PROB_RANDOM_FLAG)

// all vehicle data 

#define SITE_VEHICLE_MASK (SITE_CONST_VEHICLE_MASK | SITE_VAR_VEHICLE_MASK)    

#define SITE_TEMP_MASK    SITE_TMP_NO_CHANGE_LEFT
#define SITE_SITE_MASK    (SITE_EXTRA)

#define SITE_OCCUPIED_MASK (SITE_VEHICLE | SITE_BLOCKED)

#define SITE_DATA_MASK (SITE_VEHICLE_MASK | SITE_SITE_MASK) // all non-temporary data

// defines for extraction of data

#define SITE_GET_SITE(Site) ((Site) & SITE_SITE_MASK)
#define SITE_GET_VEHICLE(Site) ((Site) & SITE_VEHICLE_MASK)
#define SITE_GET_CONST_VEHICLE(Site) ((Site) & SITE_CONST_VEHICLE_MASK)
#define SITE_GET_VAR_VEHICLE(Site) ((Site) & SITE_VAR_VEHICLE_MASK)
#define SITE_GET_VELOCITY(Site) ((Site) & SITE_VELOCITY_MASK)
#define SITE_GET_V_MAX(Site) (((Site) & SITE_V_MAX_MASK) >> SITE_V_MAX_SHIFT)
#define SITE_GET_V_OLD(Site) (((Site) & SITE_V_OLD_MASK) >> SITE_V_OLD_SHIFT)
#define SITE_GET_DELAY(Site) (((Site) & SITE_DELAY_MASK) >> SITE_DELAY_SHIFT)
#define SITE_GET_V_OLD2(Site) (((Site) & SITE_V_OLD2_MASK) >> SITE_V_OLD2_SHIFT)
#define SITE_GET_BND_LANE(Site) (((Site) & SITE_BND_LANE_MASK) >> SITE_BND_LANE_SHIFT)

// boolean checks 

#define SITE_IS_VEHICLE(Site) ((Site) & SITE_VEHICLE)
#define SITE_IS_OCCUPIED(Site) ((Site) & SITE_OCCUPIED_MASK)
#define SITE_IS_REROUTED(Site) ((Site) & SITE_REROUTED_VEHICLE)
#define SITE_IS_DEC_PROB_RANDOM(Site) ((Site) & SITE_DEC_PROB_RANDOM_FLAG)
#define SITE_IS_HIGHLIT(Site) ((Site) & SITE_HIGHLIGHT)
#define SITE_IS_SPECIAL(Site) ((Site) &SITE_SPECIAL)
#define SITE_IS_PREV_TS_LANE_CHANGE(Site) ((Site) & SITE_PREV_TS_LANE_CHANGE)
#define SITE_CHANGE_LEFT_PERMITTED(Site) (((Site) & SITE_NO_CHANGE_LEFT) == 0)
#define SITE_CHANGE_RIGHT_PERMITTED(Site) (((Site) & SITE_NO_CHANGE_RIGHT) == 0)
#define SITE_IS_MARKED(Site) ((Site) & SITE_MARK_FLAGS)

// defines for modification of data

#define SITE_REMOVE_VEHICLE(Site) Site = SITE_GET_SITE(Site)
#define SITE_PUT_VEHICLE(ToSite, FromSite) ToSite = SITE_GET_SITE(ToSite) | SITE_GET_VEHICLE(FromSite)
#define SITE_SET_V_MAX(Site, Vmax) Site = (((Site) & ~(SITE_V_MAX_MASK)) | (Vmax) << SITE_V_MAX_SHIFT)
#define SITE_SET_DELAY(Site, Delay) Site = (((Site) & ~(SITE_DELAY_MASK)) | ((Delay) << SITE_DELAY_SHIFT))
	    
// defines for insertion of data

#define SITE_SHIFT_V(V) (V)
#define SITE_SHIFT_DEC_PROB_RANDOM(Random) ((Random)? SITE_DEC_PROB_RANDOM_FLAG:0)
#define SITE_SHIFT_V_MAX(Vmax) ((Vmax) << SITE_V_MAX_SHIFT)
#define SITE_SHIFT_BND_LANE(Lane) ((Lane) << SITE_BND_LANE_SHIFT)
#define SITE_SHIFT_GAP(Gap) ((Gap) << 7)

// ***TYPES***************************************************************************

typedef long TSite;
typedef TSite *const PSite;
typedef TSite *const *const PPSite;

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
