/*      file     : TrafficData/TAccessoryHandler.C
        project  : PAMINA III
        author   : Marcus Rickert
        RCS      : $Id: TAccessoryHandler.C,v 1.4 2005/03/25 12:28:51 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include <TAsciiDataTypeReader.h>

#include <TBaseStream.h>
#include <TTransferInfo.h>
#include <TEdge.h>
#include <TObjectManager.h>

#include "TAccessory.h"

#include "TAccessoryHandler.h"

// ***GLOBAL VARIABLES****************************************************************  

TClassInfo *TAccessoryHandler::ClassInfo = 0;

// ***METHODS************************************************************************* 

// =================================================================================== 
#define ParentClass TParallelObject
// =================================================================================== 

TAccessoryHandler::TAccessoryHandler(TParallelID theParallelID) :

ParentClass(ClassInfo, theParallelID)

{
    Accessories = new TAccessoryList;
}

// =================================================================================== 

TAccessoryHandler::~TAccessoryHandler(void)

{
    Accessories->DeleteAll(TRUE);
    DeleteObject(Accessories);
}

// =================================================================================== 

bool
TAccessoryHandler::Activate(const TDistributedRange &Range)

{
    TAccessoryEnum CAccessory;
    
    CAccessory.WalksForwardThrough(Accessories);
    while (CAccessory())
    {
        if (CAccessory->IsInRange(&Range))
        {
            if (!CAccessory->Activate())
            {
                Error("Activate: while activating accessory");
                return FALSE;
            }
        }
        else
        {
            //      Print("deleting inactive %s", 
            //            (char *) CAccessory->GetObjectName());
            delete (TAccessory *) CAccessory;
        }
        CAccessory++;
    }
    return TRUE;
}

// =================================================================================== 

bool
TAccessoryHandler::Decode(TBaseStream *aStream, TTransferInfo *TransferInfo)

{
    if (!ParentClass::Decode(aStream, TransferInfo))    
        return FALSE;
    if (TransferInfo->TransferMode == TM_ConstData)
        if (!aStream->Decode(AdjacenceInfoID))
            return FALSE;
    if (!ObjectManager->ReceiveObjectList((TParallelObjectList *) Accessories, 
                                          aStream, TRUE, TransferInfo))
    {
        Error("cannot receive accessories");
        return FALSE;
    }
    return TRUE;
}

/* =================================================================================== */

bool
TAccessoryHandler::DisposeRange(TDistributedRange *Range)

{
    TAccessoryEnum CAccessory;
    
    CAccessory.WalksForwardThrough(Accessories);
    while (CAccessory())
    {
        if (CAccessory->IsInRange(Range))
        {
            TAccessory *Temp = CAccessory;

            //      Print("DisposeRange: delete accessory %s", 
            //            (char *) CAccessory->GetInstanceName());
            delete Temp;
        }
        CAccessory++;
    }
    return TRUE;
}

// ===================================================================================

bool
TAccessoryHandler::Encode(TBaseStream *aStream, TTransferInfo *TransferInfo) const

{
    if (!ParentClass::Encode(aStream, TransferInfo))    
        return FALSE;
    if (TransferInfo->TransferMode == TM_ConstData)
    {
        if (!aStream->Encode(AdjacenceInfoID))
            return FALSE;

        //      Print("Encode: range %f-%f", TransferInfo->TransferRange.GetRangeBegin(),
        //            TransferInfo->TransferRange.GetRangeEnd());
    }
    if (!ObjectManager->SendObjectList((TParallelObjectList *) Accessories, 
                                       aStream, TransferInfo))
    {
        Error("cannot send accessories");
        return FALSE;
    }
    return TRUE;
}

// =================================================================================== 

TString
TAccessoryHandler::GetClassName(void) const

{
    return "TAccessoryHandler";
}

// =================================================================================== 

TString
TAccessoryHandler::GetInstanceName(void) const

{
    char Line[256];

    sprintf(Line, "(%s) NrOfA:%ld", (char *) ParentClass::GetInstanceName(),
            Accessories->GetNrOfElements());
    return Line;
}

// =================================================================================== 

bool
TAccessoryHandler::LoadAccessories(TInfoID theAdjacenceInfoID, 
                                   TAsciiDataTypeReader *Reader)

{    
    TAccessory   *Accessory;
    long          CAccessory;
    long          Count;
    TEdge        *Edge;

    AdjacenceInfoID = theAdjacenceInfoID;
    if ((Edge = (TEdge *) GetContainer()) == 0)
    {
        Error("LoadAccessories: cannot retrieve edge");
        return FALSE;
    }
    if (!Reader->Select(GetParallelID(), Count))
    {
        Error("AssignAccessories: cannot select");
        return FALSE;
    }
    for (CAccessory = 0; CAccessory < Count; CAccessory++)
    {
        if ((Accessory = (TAccessory *) Reader->GetSelected(CAccessory)) == 0)
        {
            Error("AssignAccessories: cannot retrieve acceesory");
            return FALSE;
        }
        Accessory->ComputeRelativeLocation(Edge, AdjacenceInfoID);
        Accessories->Append(Accessory);
    }
    return TRUE;
}

// =================================================================================== 
#undef ParentClass
