/*      file     : TrafficData/TGridStat.h
        project  : PAMINA III
        author   : Marcus Rickert
        RCS      : $Id: TGridStat.h,v 1.6 2005/03/25 12:28:51 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TGRIDSTAT_H
#define TGRIDSTAT_H

/* ***INCLUDES************************************************************************ */

#include <TDLList.h>
#include <TStatObject.h>
#include <CAConfig.h>

// ***FORWARDS************************************************************************

class TBaseStream;
class TMultilaneGrid;

/* ***CLASSES************************************************************************* */

class TGridStat : public TStatObject

{
 public:

    TGridStat(TParallelID theParallelID = INVALID);
    virtual ~TGridStat(void);
    
    bool
    Collect(const TMultilaneGrid *Grid, long theAverageDelay);

    bool
    GetConsistentTravelTime(long &Count, double &TravelTime) const;

    bool
    GetConsistentVMean(double &VMean) const;

    inline long
    GetNrOfSites(void) const;

    inline long 
    GetNrOfVehicles(void) const;

    inline long
    GetSumV(void) const;
    
    inline long
    GetSumV2(void) const;
    
    inline double
    GetDensity(void) const;

    inline double
    GetFlow(void) const;

    inline double
    GetFreeSpeed(void) const;

    inline double
    GetLengthInMeters(void) const;

    double
    GetVDelta(void) const;

    inline double
    GetVMean(void) const;

    bool
    WriteTransimsLinkStat(TBaseStream *Stream);

    static TClassInfo *ClassInfo;
    static bool NewTransimsFeedback;

 private:


    virtual bool
    Accumulate(const TStatObject *StatObject);

    virtual bool
    Decode(TBaseStream *aStream, TTransferInfo *TransferInfo);

    virtual bool
    Encode(TBaseStream *aStream, TTransferInfo *TransferInfo) const;

    virtual TString
    GetClassName(void) const;

    virtual void
    Reset(void);

    //  <<< DO NOT SEPERATE <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
#   define GRIDSTAT_LONG 7
    long Length;
    long Sites;
    long Count;
    long SumV;
    long SumV2;
    long AverageDelay;
    long SampleCount;
#   define GRIDSTAT_FLOAT 1
    float FreeSpeed;
    //  >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
};

// ***TYPES***************************************************************************

typedef TDLList< TGridStat > TGridStatList;
typedef TDLListEnum< TGridStat > TGridStatEnum;

// ***INLINE METHODS******************************************************************

#ifndef NO_INLINE
#  include "TGridStat.i.C"
#endif

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
