/*      file     : TrafficData/TRouter.h
        project  : PAMINA III
        author   : Marcus Rickert
        RCS      : $Id: TRouter.h,v 1.9 2005/03/25 12:28:52 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TROUTER_H
#define TROUTER_H

// ***INCLUDES************************************************************************ 

#include <stdio.h>
#include <unistd.h>

#include <TLocation.h>
#include <TBaseObject.h>
#include <TObjectHandler.h>
#include <TStatHandler.h>
#include <TMessageRecipient.h>
#include <TRouteplan.h>

// ***FORWARDS************************************************************************

class TParser;
class TVehicle;
class TMultilaneStat;
class TRouteStep;
class TSamplingDescription;
class TShortestPath;
class TStream;
class TRoutingStat;

#include <TNode.forward.h>
#include <TEdge.forward.h>
#include <TGridStat.forward.h>

// ***DEFINES************************************************************************* 

#define ROUTE_OK 0
#define ROUTE_DIRECTION_UNDECLARED 1
#define ROUTE_TOO_SHORT 2
#define ROUTE_NO_COMMON_NODE 3
#define ROUTE_MISSING_ACCESSORY 4
#define ROUTE_ERRORS 5

#define MAX_ROUTE_LENGTH 2048

// ***TYPES***************************************************************************

extern "C" 
struct TRouteInfo

{
    long SourceID;
    long DepartureTime;
    long Offset;
    long Length;
};
    
// ***CLASSES************************************************************************* 

class TRouter : public TBaseObject, public TObjectHandler, TMessageRecipient,
                public TStatHandler

{
public:

    TRouter(void);
    virtual ~TRouter(void);

    bool
    Activate(void);

    bool
    AreRoutesPending(void) const;

    bool
    CloseFiles(void);
    
    bool
    ClipTransimsRoute2Pamina(LongID PlanID, long MinTime, long MaxTime,
                             LongID SourceAccessory, LongID DestAccessory,
                             TRouteStep *TransimsEntries, int NrOfTransimsLinks, 
                             TEdge **PaminaLinks, int &NrOfPaminaLinks, 
                             int &FirstPaminaLink, short int &RouteplanFlags, 
                             LongID &SourceID, LongID &DestID);

    bool
    ExecuteSubTimeStep(const TTimeStep &TimeStep);

    double
    GetCompletion(void);

    TString
    GetErrorString(void) const;

    bool
    HandleExecutedRouteplan(const TTimeStep &TimeStep, TRouteplan *Routeplan);

    bool
    HandleNewRouteplan(const TTimeStep &TimeStep, TRouteplan *Routeplan);

    bool
    OpenRouteplanForWriting(bool SortByDepartureTime = TRUE);

    bool
    ReadTransimsPlan(void);

    void
    Reset(void);

    bool
    Run(const TTimeStep &StartTimeStep);

    bool
    SortRouteplans(void) const;

    bool
    WriteRoute(TVehicle *Vehicle);

    bool
    WriteLog(const char *Format, ...);

    bool
    WriteStat(const char *Format, ...);
    
    bool
    WriteStatistics(void);
    
private:

    bool
    ActivateLocalRerouting(void);

    bool
    ActivateStatistics(void);

    bool
    CheckRouteplans(TTimeStep TimeStep);

    bool
    CloseRouteplanInput(void);

    virtual bool
    Collect(TSamplingDescription *Description);

    bool
    CollectRoutingStat(TRoutingStat *Stat);

    bool
    EstimateRemainingTravelTimes(const TTimeStep &CurrentTime);

    bool
    GetAccessory(TEdge *Edge, LongID TransimsAccessoryID, LongID &PaminaAccessoryID);

    virtual TString
    GetClassName(void) const;

    bool
    GetLocation(TParallelID ID, bool IsAccessory, TLocation &Location);

    virtual bool
    HandleAggregationResult(TAggregationInfo *AggregationInfo, 
                            TTimeStep &TimeStep);

    virtual bool
    HandleAnonymousAction(TClassInfo *ClassInfo, TObjectAction Action);

    virtual bool
    HandleCreateRequest(TParallelObject *&Object, const TClassInfo *ClassInfo, 
                        TParallelID ParallelID);

    virtual bool
    HandleEvent(TEventID EventID, TBaseMessage *Message, int &Handled);

    bool
    HandleEventCompletedRoute(TBaseMessage *Message);

    bool
    HandleGridResult(TAggregationInfo *AggregationInfo, TTimeStep &TimeStep);

    bool
    HandleRoutingResult(TAggregationInfo *Info, TTimeStep &TimeStep);

    virtual bool
    HandleLoadRequest(TParallelObject *Object, bool &Loaded);

    bool
    IsInReroutingSelection(TRouteplan *Route);

    bool
    InitRoutesByOrigin(void);

    bool
    OpenCompletedRoutesOutput(void);

    bool
    OpenConversionLogFile(void);

    bool
    OpenConversionStatFile(void);

    bool
    OpenTransimsLinkStatOutput(void);

    bool
    OpenRouteplanForReading(void);

    bool
    OpenRoutesByOriginOutput(void);

    bool
    ReadRoute(long DepartureTimeStep);

    bool
    Reroute(long CurrentTimeStep, TParallelID CurrentEdgeID,
            double RelativeLocation, TVehicle *Vehicle, int LastValidStep,
            long EstimatedArrivalTimeStep);

    bool
    StoreGridStats(TClassInfo *StatClassInfo, TGridStatList *StatList);

    bool
    SendExecutedRouteplans(void);

    bool
    UpdateGridStatDestList(void);

    bool
    WriteReroutingLog(const char *Format, ...);

    bool
    WriteRouteOriginStat(TRouteplan *Routeplan);

    bool
    WriteTransimsLinkStat(TGridStatList *StatList);

    TTimeStep StartTimeStep;

    TString  RouteplanBasename;
    TString  RouteplanFilename;
    TString  RouteplanExtension;
    TStream *RouteplanOutput;
    TStream *RouteplanTempOutput;
    TStream *CompletedRoutesOutput;
    TStream *RoutesByOriginOutput;

    bool        BinaryRouteplan;
    TRouteplan *SkipRouteplan;
    TVehicle   *SkipVehicle;

    TString  OffsetFilename;
    TString  RouteplanTempFilename;

    TParser *RouteplanParser;
    long     CurrentRouteplanTimeStep;

    FILE    *ConversionLogOutput;
    long     ConversionLogEntries;
    FILE    *ConversionStatOutput;


    bool     SortByDepartureTime;
    FILE    *OffsetFile;
    long     OffsetFileHeaderSize;
    long     RoutesWritten;

    long     ErrorNodeID, ErrorEdgeID, SecondErrorEdgeID;
    long     ErrorID;

    long     RoutesProcessed;
    long     RoutesValid;
    long     RoutesCorrupted;
    long     RoutesOutsideTimeWindow;
    long     RoutesOutsideRegion;
    long     RoutesStartingOutsideRegion;
    long     RoutesEndingOutsideRegion;
    long     MinRouteLength, MaxRouteLength, SumRouteLength;

    TRouteplanList         ExecutedRouteplans;

    TSamplingDescription  *GridSamplingDescription;
    TAggregationInfo      *GridAggregationInfo;
    long                   GridStatSampleInterval;
    long                   GridStatCollectInterval;
    TStream               *TransimsLinkStatOutput;
    int                    MaxGridStatEntries;

    TSamplingDescription  *LocalGridSamplingDescription;
    TAggregationInfo      *LocalGridAggregationInfo;
    TCPNInfoList           LocalGridDestList;
    double                 LocalGridStatOverlap;

    TRoutingStat          *LatestRoutingStat;
    TSamplingDescription  *RoutingSamplingDescription;
    TAggregationInfo      *RoutingAggregationInfo;
    int                    RoutingStatInterval;
    TStream               *RoutingStatOutput;
    bool                   RoutingStatDump;
    double                 Completion;

    TRectangle             RouteDestRectangle;
    TLocation              RouteDestLocation;
    int                    RouteOriginSectors;

    bool                   LocalGridStatValid;
    TShortestPath         *LocalSP;
    TStream               *ReroutingLog;
    int                    TravelTimeEstimateUpdateInterval;
    double                 ReroutingFraction;
    int                    ReroutingFractionLimit;
    bool                   ReroutingLogActive;
    double                 ReroutingMinRelativeDelay;
    double                 ReroutingMinRelativeImprovement;
};

// ***EXTERNALS*********************************************************************** 

extern TRouter *Router;

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
