/*      file     : TrafficData/TRoutingStat.C
        project  : PAMINA III
        authors  : Marcus Rickert
        RCS      : $Id: TRoutingStat.C,v 1.5 2005/03/25 12:28:52 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include <stdio.h>

#include <TBaseStream.h>
#include <TTimeStep.h>
#include <TStream.h>
#include <TSource.h>
#include <TSink.h>
#include <TRoutingStat.h>

// ***GLOBAL VARIABLES****************************************************************  

TClassInfo *TRoutingStat::ClassInfo = 0;

// ***METHODS************************************************************************* 

#ifdef NO_INLINE
#include <TRoutingStat.i.C>
#endif

// =================================================================================== 
#define ParentClass TStatObject
// =================================================================================== 

TRoutingStat::TRoutingStat(TParallelID theParallelID) : 

ParentClass(ClassInfo, theParallelID)

{
    Reset();
}

// =================================================================================== 

TRoutingStat::~TRoutingStat(void)

{
}

// =================================================================================== 

bool
TRoutingStat::Accumulate(const TStatObject *StatObject)

{
    TRoutingStat *RoutingStat = (TRoutingStat *) StatObject;

    if (!ParentClass::Accumulate(StatObject))
        return FALSE;
    NrOfSources += RoutingStat->GetNrOfSources();
    NrOfSinks += RoutingStat->GetNrOfSinks();
    NrOfVehicles += RoutingStat->GetNrOfVehicles();
    OpenRouteplanFiles += RoutingStat->GetOpenRouteplanFiles();
    PendingVehicles += RoutingStat->GetPendingVehicles();
    Deletions += RoutingStat->GetDeletions();
    DelayedInsertions += RoutingStat->GetDelayedInsertions();
    NormalInsertions += RoutingStat->GetNormalInsertions();
    RoutesFailed += RoutingStat->RoutesFailed;
    SumDelay += RoutingStat->SumDelay;
    return TRUE;
}

// =================================================================================== 

bool
TRoutingStat::Decode(TBaseStream *aStream, TTransferInfo *TransferInfo)

{
    return (ParentClass::Decode(aStream, TransferInfo) AND
            aStream->Decode(NrOfSources, ROUTING_STAT_LONG));
}

// =================================================================================== 

bool
TRoutingStat::Encode(TBaseStream *aStream, TTransferInfo *TransferInfo) const

{
    return (ParentClass::Encode(aStream, TransferInfo) AND
            aStream->Encode(NrOfSources, ROUTING_STAT_LONG));
}

// =================================================================================== 

void
TRoutingStat::PrintScreenStat(FILE *OutputFile)

{
    fprintf(OutputFile, "Routing statistics of time step %s\n", 
            (char *) GetTimeStep().GetInstanceName());
    fprintf(OutputFile, 
            "Open Files:%ld, Ins: %ld (delayed: %ld), Pending:%ld, Del: %ld, Failed:%ld, "
            "Sources: %ld, Sinks:%ld\n",
            GetOpenRouteplanFiles(), GetInsertions(), GetDelayedInsertions(), 
            GetPendingVehicles(), GetDeletions(), GetRoutesFailed(), 
            GetNrOfSources(), GetNrOfSinks());
}

// =================================================================================== 

void
TRoutingStat::Reset(void)

{
    OpenRouteplanFiles =0;

    NrOfSources = 0;
    NrOfSinks = 0;
    NrOfVehicles = 0;
    PendingVehicles = 0;
    NormalInsertions = 0;
    DelayedInsertions = 0;
    Deletions = 0;
    RoutesFailed = 0;
    SumDelay = 0;
}

// =================================================================================== 

bool
TRoutingStat::Write(const TTimeStep &BaseTime, TStream *Stream)

{
    return Stream->Write("%ld %ld %f %ld %ld %ld %ld %ld %ld %ld %ld\n",
                         (long) GetTimeStep(),
                         (long) BaseTime + (long) GetTimeStep(),
                         (double) ((long) BaseTime + (long) GetTimeStep()) / 3600.0,
                         GetNrOfRoutedVehicles(),
                         NormalInsertions,
                         DelayedInsertions,
                         Deletions,
                         PendingVehicles,
                         OpenRouteplanFiles,
                         RoutesFailed,
                         SumDelay);
}

// =================================================================================== 

bool
TRoutingStat::WriteStatDescription(TStream *Stream)

{
    int Col = 0;

    Stream->Write("# column description\n");
    Stream->Write("# %2d : simulation timestep\n", ++Col);
    Stream->Write("# %2d : application timestep\n", ++Col);
    Stream->Write("# %2d : application time\n", ++Col);
    Stream->Write("# %2d : vehicles in system\n", ++Col);
    Stream->Write("# %2d : normal insertions\n", ++Col);
    Stream->Write("# %2d : delayed insertions\n", ++Col);
    Stream->Write("# %2d : deletions\n", ++Col);
    Stream->Write("# %2d : pending vehicles\n", ++Col);
    Stream->Write("# %2d : open routeplan files\n", ++Col);
    Stream->Write("# %2d : routes failed\n", ++Col);
    Stream->Write("# %2d : sum delay\n", ++Col);
    return TRUE;
}

// =================================================================================== 
#undef ParentClass
