/*      file     : TrafficData/TVehicle.C
        project  : PAMINA III
        author   : Marcus Rickert
        RCS      : $Id: TVehicle.C,v 1.7 2005/03/25 12:28:52 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include <stdio.h>

#include <TParser.h>
#include <TLocation.h>
#include <TBaseStream.h>
#include <TTransferInfo.h>

#include "TRouteplan.h"
#include "TVehicle.h"

// ***DEFINES************************************************************************* 

#define VEHICLE_DEFAULT_MAX_VELOCITY   120.0 // [km/h]

// ***GLOBAL VARIABLES****************************************************************  

TClassInfo *TVehicle::ClassInfo = 0;
TDLList< TVehicle > TVehicle::UnusedVehicles;

// ***METHODS************************************************************************* 

#ifdef NO_INLINE
#  include <TVehicle.i.C>
#endif

// =================================================================================== 
#define ParentClass TParallelListObject
// =================================================================================== 

TVehicle::TVehicle(TParallelID theParallelID) :

ParentClass(ClassInfo, theParallelID)

{
    _Init();
}

// =================================================================================== 

TVehicle::TVehicle(TRouteplan *theRouteplan, TBaseStream *Stream, TTransferInfo *TransferInfo) : 

ParentClass(ClassInfo, theRouteplan->GetRouteID())

{
    _Init();
    Routeplan = theRouteplan;
    SetBaseFlags(V_FLAG_ROUTEPLAN);
    if (!Decode(Stream, TransferInfo))
    {
        Fail("cannot decode");
    }
    CurrentVelocity = 0.0;
}

// =================================================================================== 

TVehicle::~TVehicle(void)

{
    DeleteObject(Routeplan);
}

// ===================================================================================

TBaseObject *
TVehicle::Copy(void) const

{
    TVehicle *NewVehicle;

    NewVehicle = new TVehicle(GetParallelID());
    NewVehicle->SetMaxVelocity(MaxVelocity);
    return NewVehicle;
}

// =================================================================================== 

bool
TVehicle::Decode(TBaseStream *aStream, TTransferInfo *TransferInfo)

{
    if (!ParentClass::Decode(aStream, TransferInfo))
        return FALSE;
    switch(TransferInfo? TransferInfo->TransferMode : TM_VarData)
    {
    case TM_AsciiData:
        
        if (!aStream->Decode(MaxVelocity))
            return FALSE;
        break;
        
    case TM_VarData:
    case TM_ConstData:
        
        if (!(aStream->Decode(Lane, VEHICLE_LONG) AND
              aStream->Decode(CurrentVelocity, VEHICLE_FLOAT)))
            return FALSE;
        if (AreBaseFlagsSet(V_FLAG_ROUTEPLAN))
        {
            Routeplan = new TRouteplan(aStream, TransferInfo);
            if (Routeplan->Failed())
                return FALSE;
        }
        else
            Warning("haha!");
        break;

    default:

        debug(Print(DM_CODE, "Decode: TransferMode?"));
    }
    return TRUE;
}

// =================================================================================== 

bool
TVehicle::Encode(TBaseStream *aStream, TTransferInfo *TransferInfo) const

{
    if (!ParentClass::Encode(aStream, TransferInfo))
        return FALSE;
    switch(TransferInfo? TransferInfo->TransferMode : TM_VarData)
    {
    case TM_AsciiData:
        
        if (!aStream->Encode(MaxVelocity))
            return FALSE;
        break;

    case TM_VarData:
    case TM_ConstData:

        if (!(aStream->Encode(Lane, VEHICLE_LONG) AND
              aStream->Encode(CurrentVelocity, VEHICLE_FLOAT)))     
            return FALSE;
        if (Routeplan)
        {
            if (!Routeplan->Encode(aStream, TransferInfo))
                return FALSE;
        }
        break;

    default:

        debug(Print(DM_CODE, "Encode: TransferMode?"));
    }
    return TRUE;
}

// =================================================================================== 

TSite
TVehicle::GetCAVelocity(void) const

{
    return (TSite) REAL_VELOCITY_TO_CA_VELOCITY(CurrentVelocity);
}

// =================================================================================== 

TString
TVehicle::GetClassName(void) const

{
    return "TVehicle";
}

// =================================================================================== 

TString
TVehicle::GetInstanceName(void) const

{
    char Line[128];

    sprintf(Line, "V:%.1f MaxV:%.1f", CurrentVelocity, MaxVelocity);
    return Line;
}

// ===================================================================================

TSite
TVehicle::GetMaxCAVelocity(void) const

{
    return (TSite) REAL_VELOCITY_TO_CA_VELOCITY(MaxVelocity);
}

// =================================================================================== 

TSite
TVehicle::GetSitePattern(void) const

{
    long Site;

    Site = (SITE_VEHICLE | 
            SITE_SHIFT_V(GetCAVelocity()) |
            SITE_SHIFT_V_MAX(GetMaxCAVelocity()));
    if (TraceActive())
        Site |= SITE_HIGHLIGHT;
    return Site;
}

// =================================================================================== 

void
TVehicle::_Init(void)

{
    Flags = 0;
    CurrentVelocity = 0.0;
    MaxVelocity = 0.0;
    Routeplan = 0;
}

// =================================================================================== 

void
TVehicle::SetSitePattern(long Site)

{
    CurrentVelocity = CA_VELOCITY_TO_REAL_VELOCITY(SITE_GET_VELOCITY(Site));
}

// =================================================================================== 
#undef ParentClass

