/*      file     : common/TArray.C
        project  : General Toolbox
        author   : Marcus Rickert
        RCS      : $Id: TArray.C,v 1.10 2005/03/25 11:04:59 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

/* ***INCLUDES************************************************************************ */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <TDebug.h>
#include <TArray.h>

/* ***METHODS************************************************************************* */

#ifdef NO_INLINE
#include <TArray.i.C>
#endif

// ===================================================================================

#ifdef DARRAY_VAR_ITEMSIZE
TArrayPrimitive::TArrayPrimitive(int theItemSize, int theIncrement, char *)
#else
TArrayPrimitive::TArrayPrimitive(int theIncrement, char *)
#endif

{
#ifdef DARRAY_VAR_ITEMSIZE
    ItemSize = theItemSize;
#endif
    Increment = theIncrement;
    Items = 0;
    LogicalNrOfItems = 0;
    DeltaItems = 0;
#ifdef DARRAYNAME
    Name = theName;
#endif
}

/* =================================================================================== */

TArrayPrimitive::~TArrayPrimitive(void)

{
    Deallocate();
}

/* =================================================================================== */

bool
TArrayPrimitive::ActionAfterInsertion(long, long)

{
    return TRUE;
}

/* =================================================================================== */

bool
TArrayPrimitive::ActionBeforeDeletion(long, long)

{
    return TRUE;
}

/* =================================================================================== */

bool
TArrayPrimitive::_Add(void *Item)

{
    if (GetNrOfItems() == GetPhysicalNrOfItems())
        if (!Enlarge())
            return FALSE;
    _Set(LogicalNrOfItems, Item);
    SetLogicalNrOfItems(LogicalNrOfItems + 1);
    return TRUE;
}

/* =================================================================================== */

bool
TArrayPrimitive::Allocate(long theNrOfItems)

{
    
    if (theNrOfItems != GetPhysicalNrOfItems())
    {
        if  (Items)
            if (theNrOfItems)
            {
                if ((Items = (char *) realloc(Items, theNrOfItems*GetItemSize())) == 0)
                {
                    Error("Allocate: realloc returns out of heap");
                    return FALSE;
                }
            }
            else
            {
                Free(Items);
            }
        else
            if (theNrOfItems)
                if ((Items = (char *) malloc(theNrOfItems*GetItemSize())) == 0)
                {
                    Error("Allocate: malloc returns out of heap");
                    return FALSE;
                }
        DeltaItems = (short) (theNrOfItems - LogicalNrOfItems);
    }
    return TRUE;
}
            
/* =================================================================================== */

void
TArrayPrimitive::Delete(long Index)

{
    long i;
    long TempItemSize = GetItemSize();
    
    ActionBeforeDeletion(Index, Index);
    for (i = Index; i < LogicalNrOfItems-1; i++)
        memcpy(Items + i * TempItemSize, Items + (i + 1) * TempItemSize, TempItemSize);
    SetLogicalNrOfItems(LogicalNrOfItems - 1);
}

/* =================================================================================== */

void
TArrayPrimitive::Deallocate(void)

{
    Allocate(0);
}

/* =================================================================================== */

bool
TArrayPrimitive::Enlarge(void)

{
    return Allocate(GetPhysicalNrOfItems() + Increment);
}

/* =================================================================================== */

TString
TArrayPrimitive::GetInstanceName(void) const

{
    char line[128];

#ifdef DARRAYNAME
    sprintf(line, "Name:%s #P:%ld #L:%ld", ((char *) Name)? (char *) Name : "(null)",
            GetPhysicalNrOfItems(), LogicalNrOfItems);
#else
    sprintf(line, "#P:%ld #L:%ld", GetPhysicalNrOfItems(), LogicalNrOfItems);
#endif
    return line;
}

/* =================================================================================== */
#ifndef DARRAY_VAR_ITEMSIZE
long
TArrayPrimitive::GetItemSize(void) const

{
    NotOverloaded("GetItemSize");
    return -1;
}
#endif
/* =================================================================================== */

bool
TArrayPrimitive::_InsertAt(long Index, void *Data)

{
    long TempItemSize = GetItemSize();

    if (GetNrOfItems() == GetPhysicalNrOfItems())
        if (!Enlarge())
            return FALSE;
    SetLogicalNrOfItems(LogicalNrOfItems + 1);
    if (LogicalNrOfItems - Index - 1 > 0)
        memcpy(Items + (Index + 1) * TempItemSize, Items + Index  * TempItemSize,
               (LogicalNrOfItems - Index - 1) * TempItemSize);
    _Set(Index, Data);
    return TRUE;
}

/* =================================================================================== */

bool
TArrayPrimitive::Resize(long aNrOfItems)

{
    long OldLogicalNrOfItems;

    OldLogicalNrOfItems = LogicalNrOfItems;
    if (aNrOfItems < LogicalNrOfItems)
        if (!ActionBeforeDeletion(aNrOfItems, LogicalNrOfItems - 1))
            return FALSE;
    if (aNrOfItems < GetPhysicalNrOfItems() - Increment OR 
        aNrOfItems > GetPhysicalNrOfItems())
        if (!Allocate(aNrOfItems))
            return FALSE;
    SetLogicalNrOfItems(aNrOfItems);
    if (aNrOfItems > OldLogicalNrOfItems)
        if (!ActionAfterInsertion(OldLogicalNrOfItems, aNrOfItems - 1))
            return FALSE;
    return TRUE;
}

/* =================================================================================== */

void
TArrayPrimitive::_Set(long Index, void *Data)

{
    long TempItemSize = GetItemSize();

    memcpy((void *) (Items + Index * TempItemSize), Data, TempItemSize);
}

/* =================================================================================== */

void
TArrayPrimitive::SetLogicalNrOfItems(long theNumber)

{
    DeltaItems += (LogicalNrOfItems - theNumber);
    LogicalNrOfItems = theNumber;
}

/* =================================================================================== */

