/*      file     : common/THeap.C
        project  : General Toolbox
        author   : Marcus Rickert
        RCS      : $Id: THeap.C,v 1.7 2005/03/25 11:04:59 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include "THeap.h"

// ***METHODS************************************************************************* 

#ifdef NO_INLINE
#  include <THeap.i.C>
#endif

// =================================================================================== 

int
THeapObject::HeapCompare(THeapObject *, int) const

{    
    return 0;
}


// =================================================================================== 
#define ParentClass TBaseObject
// =================================================================================== 

THeapPrimitive::THeapPrimitive(int theCompareCode)

{
    CompareCode = theCompareCode;
}

// =================================================================================== 

bool
THeapPrimitive::Add(THeapObject *Object)

{
    long Index;

    Elements.Add(Object);
    Index = GetNrOfElements() - 1;
    Elements[Index]->SetHeapIndex(Index);
    MoveUp(Index);
    return TRUE;
}

// =================================================================================== 

void
THeapPrimitive::DeleteAll(void)

{
    Elements.Resize(0);
}

// =================================================================================== 


THeapObject *
THeapPrimitive::GetTop(void)

{
    THeapObject *Temp;
    long         Index;

    if (GetNrOfElements() == 0)
    {
        Warning("GetHead called for empty heap");
        return 0;
    }
    Temp = Elements.Get(0);
    Elements.Get(0) = Elements.Get(GetNrOfElements() - 1);
    Elements.Get(0)->SetHeapIndex(0);
    Elements.Delete(GetNrOfElements() - 1);
    Index = 0;
    MoveDown(Index);
    Temp->SetHeapIndex(NOT_IN_HEAP);
    return Temp;
}

// =================================================================================== 

void
THeapPrimitive::MoveDown(long &Index)

{
    while ((HEAP_HAS_L_SON(Index, GetNrOfElements()) AND
            Elements.Get(Index)->HeapCompare(Elements.Get(HEAP_L_SON(Index))) < 0) OR
           (HEAP_HAS_R_SON(Index, GetNrOfElements()) AND
            Elements.Get(Index)->HeapCompare(Elements.Get(HEAP_R_SON(Index))) < 0))
        if (HEAP_HAS_R_SON(Index, GetNrOfElements()) AND
            Elements.Get(HEAP_R_SON(Index))->HeapCompare(Elements.Get(HEAP_L_SON(Index))) > 0)
        {
            THeapObject *Temp;
            
            Temp = Elements.Get(Index);
            Elements.Get(Index) = Elements.Get(HEAP_R_SON(Index));
            Elements.Get(Index)->SetHeapIndex(Index);
            Elements.Get(HEAP_R_SON(Index)) = Temp;
            Elements.Get(HEAP_R_SON(Index))->SetHeapIndex(HEAP_R_SON(Index));
            Index = HEAP_R_SON(Index);
        }
        else
        {
            THeapObject *Temp;
            
            Temp = Elements.Get(Index);
            Elements.Get(Index) = Elements.Get(HEAP_L_SON(Index));
            Elements.Get(Index)->SetHeapIndex(Index);
            Elements.Get(HEAP_L_SON(Index)) = Temp;
            Elements.Get(HEAP_L_SON(Index))->SetHeapIndex(HEAP_L_SON(Index));
            Index = HEAP_L_SON(Index);
        }
}

// =================================================================================== 

void
THeapPrimitive::MoveUp(long &Index)

{
    while (HEAP_HAS_FATHER(Index) AND
           Elements.Get(Index)->HeapCompare(Elements.Get(HEAP_FATHER(Index))) > 0)
    {
        THeapObject *Temp;
        
        Temp = Elements.Get(Index);
        Elements.Get(Index) = Elements.Get(HEAP_FATHER(Index));
        Elements.Get(Index)->SetHeapIndex(Index);
        Elements.Get(HEAP_FATHER(Index)) = Temp;
        Elements.Get(HEAP_FATHER(Index))->SetHeapIndex(HEAP_FATHER(Index));
        Index = HEAP_FATHER(Index);
    }
}

// =================================================================================== 

void
THeapPrimitive::Update(THeapObject *Object)

{
    long Index;

    Index = Object->GetHeapIndex();
    MoveUp(Index);
    MoveDown(Index);
}

// =================================================================================== 
#undef ParentClass
