/*      file     : common/THeap.h
        project  : General Toolbox
        author   : Marcus Rickert
        RCS      : $Id: THeap.h,v 1.12 2005/03/25 11:04:59 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#include "gcc_pragma_interface.h"

#ifndef THEAP_H
#define THEAP_H

// ***INCLUDES************************************************************************

#include <TBaseObject.h>
#include <TArray.h>

// ***DEFINES************************************************************************* 

#define NOT_IN_HEAP (-1)

#define HEAP_FATHER(SonNr) ((SonNr - 1) / 2)
#define HEAP_HAS_FATHER(SonNr) (SonNr > 0)
#define HEAP_L_SON(FatherNr) (2 * FatherNr + 1)
#define HEAP_HAS_L_SON(FatherNr, NrOfElements) (HEAP_L_SON(FatherNr) < NrOfElements)
#define HEAP_R_SON(FatherNr) (2 * FatherNr + 2)
#define HEAP_HAS_R_SON(FatherNr, NrOfElements) (HEAP_R_SON(FatherNr) < NrOfElements)
#define HEAP_IS_L_SON(SonNr) ((SonNr & 1) > 0)
#define HEAP_IS_R_SON(SonNr) (!HEAP_IS_L_SON(SonNr))

// ***CLASSES************************************************************************* 

/**
  */

class THeapObject

{
public:

    THeapObject(void) { HeapIndex = NOT_IN_HEAP; };

    /** Compare the current instance with another instance of the same class.

     This method must be overwritten, if the class is to be used in sorted containers.

     @param OtherObject pointer to the second innstance which may have to be type-casted.
     @param CompareCode integer that selects the sorting mode, defaults to 0
     @return <B>-1</B> if the local instance is 'less' than the other instance,
     <B>0</B> if the instances are equivalent,
     <B>1</B> if the local instance id 'greater' than the other instance
     */

    virtual int
    HeapCompare (THeapObject *OtherObject, int CompareCode = 0) const;

    void
        Dummy(void);
    inline long
        GetHeapIndex(void) const;
    inline bool
        IsInHeap(void) const;
    inline void
        SetHeapIndex(long theIndex);
    
private:

    long HeapIndex;
};

// ===================================================================================

/**
  */

class THeapPrimitive : public TBaseObject

{
 public:

    THeapPrimitive(int theCompareCode);

    void
    DeleteAll(void);

    inline long
        GetNrOfElements(void) const;
    void
        Update(THeapObject *);

 protected:   

    TArray<THeapObject *>  Elements;

    bool
        Add(THeapObject *);
    THeapObject *
        GetTop(void);

 private:

    int CompareCode;

    void
        MoveDown(long &Index);
    void
        MoveUp(long &Index);
};

// ===================================================================================

/**
  */

template < class T >
class THeap : public THeapPrimitive

{
public:
    THeap(int theCompareCode = 0);
    
    bool
        Add(T *Object);
    T *
        Get(long Index);
    T *
        GetTop(void);
};

// ***TEMPLATE METHODS****************************************************************

#if defined(TEMPLATEBODIES) || defined(INCLUDE_TEMPLATES)
#  include "THeap.t.C"
#endif

// ***INLINE METHODS****************************************************************** 

#ifndef NO_INLINE
#  include "THeap.i.C"
#endif

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
