/*      file     : common/TList.C
        project  : General Toolbox
        author   : Marcus Rickert
        RCS      : $Id: TList.C,v 1.6 2005/03/25 11:04:59 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include <stdio.h>

#include <Basic.h>
#include <TDebug.h>
#include <TArray.h>
#include <TDLList.h>

// ***FUNCTIONS*********************************************************************** 

void
InitListReference(TDLListReference &Ref)

{
    Ref.Prev = Ref.Next = 0;
    Ref.PrevIndex = Ref.NextIndex = -1; 
    Ref.List = 0;
}

// ***METHODS*************************************************************************

#ifdef NO_INLINE
#include "TDLList.i.C"
#endif

#include "TListNode.C"
#include "TSLList.C"
#include "TDLList.C"
#include "TListEnum.C"

// =================================================================================== 
#define ParentClass
// =================================================================================== 

TDLListPrimitive::TDLListPrimitive(void)

{
    int Index;

    NrOfElements = 0;
    CurrentEnum = 0;
    if (Head.GetFreeReferenceIndex(Index))
    {
        Head.SetList(Index, this);
        Head.SetPrev(Index, &Head, Index);
        Head.SetNext(Index, &Head, Index);
    }
}

/* =================================================================================== */

TDLListPrimitive::~TDLListPrimitive(void)

{
    DeleteAll(FALSE);
    if (CurrentEnum)  /* MR 1-17-94 discovered by PURIFY */
    {
        CurrentEnum->UnlinkList();
        CurrentEnum = 0;
    }
}

/* =================================================================================== */

void
TDLListPrimitive::_Absorb(TDLListPrimitive &OtherList)

{
    if (OtherList.GetNrOfElements())
    {
        TDLListEnumPrimitive  CObject;
        TListNodePrimitive   *OtherFirst, *OtherLast;
        int                   OtherLastIndex, OtherFirstIndex;
        
        CObject._WalksForwardThrough(OtherList);
        while (CObject())
        {
            CObject->SetList(CObject.GetCurrentIndex(), this);
            CObject++;
        }               
        OtherLast = (TListNodePrimitive *) OtherList.Head.GetPrev(0);
        OtherLastIndex = OtherList.Head.GetPrevIndex(0);
        OtherFirst = (TListNodePrimitive *) OtherList.Head.GetNext(0);
        OtherFirstIndex = OtherList.Head.GetNextIndex(0);
        if (NrOfElements)
        {
            TListNodePrimitive *Last;
            int                 LastIndex;

            Last = (TListNodePrimitive *) Head.GetPrev(0);
            LastIndex = Head.GetPrevIndex(0);

            Last->SetNext(0, OtherFirst, OtherFirstIndex);
            OtherFirst->SetPrev(OtherFirstIndex, Last, LastIndex);
        }
        else
        {
            Head.SetNext(0, OtherFirst, OtherFirstIndex);
            OtherFirst->SetPrev(OtherFirstIndex, &Head, 0);
        }
        Head.SetPrev(0, OtherLast, OtherLastIndex);
        OtherLast->SetNext(OtherLastIndex, &Head, 0);
        OtherList.Head.SetNext(0, &OtherList.Head, 0);
        OtherList.Head.SetPrev(0, &OtherList.Head, 0);
        NrOfElements += OtherList.NrOfElements;
        OtherList.NrOfElements = 0;
    }
}

/* =================================================================================== */

bool
TDLListPrimitive::_Append(const TListNodePrimitive *Object, bool Unique)

{
    return Object->LeftLink(this, Head.GetPrev(0), Unique);
}

/* =================================================================================== */

bool
TDLListPrimitive::_Append(TDLListPrimitive &List, bool Unique)

{
    TDLListEnumPrimitive CObject;

    CObject._WalksForwardThrough(List);
    while (CObject())
    {
        if (!_Append(CObject, Unique))
            return FALSE;
        CObject++;
    }
    return TRUE;
}

/* =================================================================================== */

bool
TDLListPrimitive::_Delete(const TListNodePrimitive *Object)

{
    return Object->Unlink(this);
}

/* =================================================================================== */

bool
TDLListPrimitive::DeleteAll(bool DisposeElements)

{
    TDLNodePrimitive     *Node;
    TDLListEnumPrimitive  CNode;

    if (NrOfElements > 0)
    {
        CNode._WalksForwardThrough(*this);
        CurrentEnum = 0;
        while (CNode())
        {
            // We are cheating here! It should be sufficient to delete a pointer
            // to TListNodePrimitive instead of casting it first to TDLNodePrimitive.
            // Not doing so, however, results in memory deallocation problems. Maybe
            // TListNodePrimitive must have a dummy virtual destructor?
            
            Node = (TDLNodePrimitive *) (TListNodePrimitive *) CNode;
            CNode++;
            if (DisposeElements)
                delete Node;
            else
                Node->Unlink(this);
        }
    }
    return TRUE;
}

/* =================================================================================== */

TString
TDLListPrimitive::GetInstanceName(void) const

{
    char line[128];

    sprintf(line, "#elements:%ld", NrOfElements);
    return line;
}

/* =================================================================================== */

bool
TDLListPrimitive::_IsInList(const TListNodePrimitive *Object) const

{
    return Object->IsInList(this);
}

/* =================================================================================== */

bool
TDLListPrimitive::_Prepend(const TListNodePrimitive *Object, bool Unique)

{
    return Object->LeftLink(this, &Head, Unique);
}

// =================================================================================== 
#undef ParentClass
