/*      file     : common/TString.h
        project  : General Toolbox
        author   : Marcus Rickert
        RCS      : $Id: TString.h,v 1.13 2005/03/25 11:05:00 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TSTRING_H
#define TSTRING_H

/* ***INCLUDES************************************************************************ */

#include <Basic.h>
#include <TBaseObject.h>

/* ***DEFINES************************************************************************* */

#define PATH_SEPERATOR '/'
#define STRCMP_NO_PREFIX   0
#define STRCMP_S1_PREFIX   1
#define STRCMP_S2_PREFIX   2
#define STRCMP_PREFIX_MASK 3
#define STRCMP_IGNORE_CASE 4

// ===================================================================================

#ifdef EMPTY_NULL_STRING
#  define _NullString 0
#else
#  define _NullString "<NULLSTRING>"
#endif

#ifdef CHECK_STRING
#define CheckEmptyS(action, retval) \
       if (S == 0) \
       { \
             Warning("method '%s' called for NULL string", action); \
             return retval; \
       }
#else
#define CheckEmptyS(action, retval) \
       if (S == 0) \
             return retval;
#endif

// ***FORWARDS************************************************************************

class TBaseStream;

// ***CLASSES*************************************************************************

/**
  */

class TString : public TBaseObject

{
  public:

    TString(char aChar);
    TString(const char *AString = _NullString);
    TString(const TString &aString);
    ~TString(void);

    int
        operator==(const TString &aTString) const;
    int
        operator==(const char *aString) const;
    int
        operator!=(const TString &aTString) const;
    int
        operator!=(const char *aString) const;
    void
        operator=(const char *aString);
    const TString &
        operator=(const TString &aString);
    char 
        operator[](int i) const;
    char * 
        operator()(void) const;

    operator char * ();
    operator const char * () const;

    void
        Consume(char *aString);
    virtual TBaseObject *
        Copy(void) const;
    /** Decode an Ascii string from the stream. */
    bool
        Decode(TBaseStream &aStream);
    /** Encode the string into the stream. */
    bool
        Encode(TBaseStream &aStream) const;
    virtual TString
        GetClassName(void) const;
    virtual TString
        GetInstanceName(void) const;
    const char *
        GetPtr(void) const;
    char *
        GetPtr(void);
    inline bool
        IsDefined(void) const;
    inline bool
        IsEmpty(void) const;
    inline int
        Length(void) const;
    TString
        SubString(int First, int Last) const;
    void
        ToLowerCase(void);
    void
        ToUpperCase(void);
    
  private:

    virtual int
        Compare(TBaseObject *OtherObject, int CompareCode) const;
    void 
        FreeString(void);
    void
        _Init(void);
    void 
        SetString(const char *AString);

    int   _Length;
    char *S;
};

/* ***PROTOTYPES********************************************************************** */

TString
    CombinePaths(const char *Path1, const char *Path2);
TString 
    GetFilenamePart(const char *aString);
TString 
    GetPathPart(const char *aString);
int
    StrCmp(const char *S1, const char *S2, int Prefix = STRCMP_NO_PREFIX);
TString
    SubString(const char *aString, int First, int Last);
TString
    operator+(const TString &S1, const TString &S2);

#ifdef __cplusplus
extern "C" 
{
#endif

#ifdef NO_STRDUP
    char *
        strdup(const char *aString);
#endif
    
#ifdef __cplusplus
}
#endif
// ***INLINE METHODS******************************************************************
#ifndef NO_INLINE
#  include "TString.i.C"
#endif
// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
