/*      file     : graphx/TWindow.h
        project  : General Toolbox
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TWindow.h,v 1.10 2005/03/25 11:06:10 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TWINDOW_H
#define TWINDOW_H

/* ***INCLUDES************************************************************************ */

#include <X11/X.h>
#include <Graphx.h>

#include <TDebug.h>
#include <TBaseObject.h>

// ***FORWARDS************************************************************************

class TLocation;

/* ***DEFINES************************************************************************* */

#define EVENT_MASK (ExposureMask | VisibilityChangeMask | StructureNotifyMask | \
                    KeyPressMask | ButtonPressMask ) 

/* ***CLASSES************************************************************************* */

class TWindow : public TBaseObject, public GrWindow

{
public:

    TWindow(const char *windowName, int Width = 500, int Height = 500,
            int ReservedColors = 0);
    virtual ~TWindow(void) ;

    //    TWindow(const char *windowName, char *theBaseFilename,
    //      int ReservedColors = 0);

    virtual bool
    Draw(void);

    void
    DrawArrow(const TLocation &From, double FromGap, const TLocation &To, double ToGap,
              double Width, bool DrawTip, int Color);

    inline void
    DumpOnNextRedraw(void);

    virtual bool
    Execute(long ID = 0);

    inline void
    MarkForRedisplay(void);

    inline void
    MarkForRedraw(void);

    virtual TString
    GetClassName(void) const;

    TString
    GetDumpLabel(void) const;

    bool
    HandleXEvents(long theDumpID = 0);

    inline bool
    Redisplay(void);

    inline bool
    Redraw(void);

    virtual void
    LabelDump(const char *Label);

    void
    SetDumpBaseFilename(const char *theBaseFilename);

    void
    WriteCentered(REAL X, REAL Y, char *S, int Color);

    inline double
    X2World(int X) const;

    inline double
    Y2World(int Y) const;

protected:

    bool RedrawFlag;
    bool RedisplayFlag;

    bool
    Dump(void);

    virtual bool
    HandleButtonPress(int x, int y, int ButtonState);

    virtual bool
    HandleKeyPress(int KeyCode);

private:

    bool    DumpActive;
    bool    DumpFlag;
    long    DumpID;
    int     DumpCounter;
    TString DumpBaseFilename;
    double  InitialDeltaX;
    double  InitialDeltaY;
    
    void
    CorrectRatios(REAL &Xmin, REAL &Ymin, REAL &Xmax, REAL &Ymax);

    bool
    Display(void);

    void
    _Init(void);
};

/* =================================================================================== */

class TCircle : public GrCircle

{
 public:

    TCircle(double X, double Y, double Radius, int Color = 0);
};

/* =================================================================================== */

class TFillCircle : public GrFillCircle

{
 public:

    TFillCircle(double X, double Y, double Radius, int Color = 0);
};

// ***INLINE METHODS******************************************************************

#ifndef NO_INLINE
#  include "TWindow.i.C"
#endif

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif

