/*      file     : graphx/grFont.c
        project  : graphical toolbox
        author   : Michael Olesen
        comments : simple classes for using X11
        RCS      : $Id: grFont.c,v 1.3 2005/03/25 11:06:10 mr Exp $
*/

#include <graphx.h>

int grLoadFont(grWindow *theWindow,int family,int weight,int shape,int size) {
  int n;
  grFont **theTempFonts;
#ifdef DEBUG
  SETFUNC("void grLoadFont(grWindow *,int,int,int)");
#endif
  
  if ((n = grGetFontNo(theWindow,family,weight,shape,size)) != -1)
    return 1;
  
  if (theWindow->noFont == 0) {
    theWindow->theFonts = 
      (grFont **)malloc(GrDefaultNoFonts*sizeof(grFont *));
    theWindow->noFont = GrDefaultNoFonts;
    theWindow->topFont = 0;
  }
  
  if (theWindow->topFont == theWindow->noFont) {
    theTempFonts = 
      (grFont **)malloc(2*theWindow->noFont*sizeof(grFont *));
    for (n = 0;n < theWindow->topFont;n++)
      theTempFonts[n] = theWindow->theFonts[n];
    for (n = 0;n < theWindow->noFont;n++)
      free(theWindow->theFonts[n]);
    free(theWindow->theFonts);
    theWindow->noFont *= 2;
    theWindow->theFonts = theTempFonts;
  }
  
  theWindow->theFonts[theWindow->topFont] =
    (grFont *)malloc(sizeof(grFont));
  theWindow->theFonts[theWindow->topFont]->family = family;
  theWindow->theFonts[theWindow->topFont]->weight = weight;
  theWindow->theFonts[theWindow->topFont]->shape = shape;
  theWindow->theFonts[theWindow->topFont]->size = size;
  n = (family == grSymbol)?(family*4*41+size):
    ((family*4+weight*2+shape)*41+size);
  if (strlen(grFontNames[n]) == 0) {
    free(theWindow->theFonts[theWindow->topFont]);
    return 0;
  }
  theWindow->theFonts[theWindow->topFont]->theFont = 
    XLoadQueryFont(theWindow->dpy,grFontNames[n]);
  if (theWindow->theFonts[theWindow->topFont]->theFont == (XFontStruct *)NULL) {
    free(theWindow->theFonts[theWindow->topFont]);
    return 0;
  }
  theWindow->topFont++;
  return 1;
}

int grLoadFontName(grWindow *theWindow,char *fontName) {
  char *str=fontName;
  int n,f,w,s,size;
#ifdef DEBUG
  SETFUNC("void grLoadFontName(grWindow *,char *)");
#endif

  for (n = 0;n < 5;n++)
    if (strncmp(str,grFontFamilies[n],strlen(grFontFamilies[n])) == 0)
      break;
      
  if (n == 5) return 0;
  
  f = n;
  str += strlen(grFontFamilies[n])+1;
  
  for (n = 0;n < 2;n++)
    if (strncmp(str,grFontWeights[n],strlen(grFontWeights[n])) == 0)
      break;
      
  if (n == 2) return 0;
  
  w = n;
  str += strlen(grFontWeights[n])+1;
  
  for (n = 0;n < 2;n++)
    if (strncmp(str,grFontShapes[n],strlen(grFontShapes[n])) == 0)
      break;
      
  if (n == 2) return 0;
  
  s = n;
  str += strlen(grFontShapes[n])+1;
  
  sscanf(str,"%d",&size);
  
  return grSetFont(theWindow,f,w,s,size);
}

void grUnloadFont(grWindow *theWindow,int family,int weight,int shape,int size) {
  int m,n;
#ifdef DEBUG
  SETFUNC("void grUnloadFont(grWindow *,int,int,int,int)");
#endif

  if ((n = grGetFontNo(theWindow,family,weight,shape,size)) == -1) return;
  
  XUnloadFont(theWindow->dpy,theWindow->theFonts[n]->theFont->fid);
  free(theWindow->theFonts[n]);
  for (m = theWindow->topFont-1;m > n;m--)
    theWindow->theFonts[m-1] = theWindow->theFonts[m];
  theWindow->topFont--;
}

void grUnloadFontName(grWindow *theWindow,char *fontName) {
  char *str=fontName;
  int n,f,w,s,size;
#ifdef DEBUG
  SETFUNC("void grUnloadFontName(grWindow *,char *)");
#endif

  for (n = 0;n < 5;n++)
    if (strncmp(str,grFontFamilies[n],strlen(grFontFamilies[n])) == 0)
      break;
      
  if (n == 5) return;
  
  f = n;
  str += strlen(grFontFamilies[n])+1;
  
  for (n = 0;n < 2;n++)
    if (strncmp(str,grFontWeights[n],strlen(grFontWeights[n])) == 0)
      break;
      
  if (n == 2) return;
  
  w = n;
  str += strlen(grFontWeights[n])+1;
  
  for (n = 0;n < 2;n++)
    if (strncmp(str,grFontShapes[n],strlen(grFontShapes[n])) == 0)
      break;
      
  if (n == 2) return;
  
  s = n;
  str += strlen(grFontShapes[n])+1;
  
  sscanf(str,"%d",&size);
  
  grUnloadFont(theWindow,f,w,s,size);
}

int grGetFontNo(grWindow *theWindow,int family,int weight,int shape,int size) {
  int n;
#ifdef DEBUG
  SETFUNC("void grGetFontNo(grWindow *,int,int,int,int)");
#endif

  if (theWindow->topFont == 0) return -1;
  for (n = 0;n < theWindow->topFont;n++) {
    if (theWindow->theFonts[n]->family == family AND
      theWindow->theFonts[n]->weight == weight AND
      theWindow->theFonts[n]->shape == shape AND
      theWindow->theFonts[n]->size == size) break;
  }
  if (n < theWindow->topFont) return n;
  return -1;
}

int grSetFont(grWindow *theWindow,int family,int weight,int shape,int size) {
  int n;
#ifdef DEBUG
  SETFUNC("void grSetFont(grWindow *,int,int,int,int)");
#endif

  if (grGetFontNo(theWindow,family,weight,shape,size) == -1) {
    if (!grLoadFont(theWindow,family,weight,shape,size)) return 0;
  }
  
  n = grGetFontNo(theWindow,family,weight,shape,size);
  XSetFont(theWindow->dpy,theWindow->gc,theWindow->theFonts[n]->theFont->fid);
  theWindow->theFont = n;
  
  return 1;
}

void grDrawChar(grWindow *theWindow,int x,int y,char c,int color) {
  char s[2];
  int xOffset=0,yOffset=0;
#ifdef DEBUG
  SETFUNC("void grDrawChar(grWindow *,int,int,char,int)");
  if (color < 0) ERROR("Trying to draw character with a negative color number");
  if (color > theWindow->noColors) WARNING("Trying to draw character with a"
    " color number higher than the actual number of colors");
#endif

  s[0] = c;s[1] = 0;
  if (theWindow->noFont == 0) return;
  if (theWindow->horizontalTextAlignment == GrCenterAlign)
    xOffset = (int) (-grGetTextWidth(theWindow,s)/2);
  if (theWindow->horizontalTextAlignment == GrRightAlign)
    xOffset = (int) (-grGetTextWidth(theWindow,s));
  if (theWindow->verticalTextAlignment == GrCenterAlign)
    yOffset = grGetTextHeight(theWindow)/2;
  if (theWindow->verticalTextAlignment == GrTopAlign)
    yOffset = grGetTextHeight(theWindow);
  XSetForeground(theWindow->dpy,theWindow->gc,
    theWindow->colors[color%theWindow->noColors].pixel);
  XDrawString(theWindow->dpy,theWindow->wd,theWindow->gc,x+xOffset,y+yOffset,s,1);
  if (theWindow->speed == GrSlow) XFlush(theWindow->dpy);
}

void grDrawString(grWindow *theWindow,int x,int y,char *s,int color) {
  int xOffset=0,yOffset=0,len=(int) strlen(s);
#ifdef DEBUG
  SETFUNC("void grDrawString(grWindow *,int,int,char *,int)");
  if (color < 0) ERROR("Trying to draw string with a negative color number");
  if (color > theWindow->noColors) WARNING("Trying to draw string with a"
    " color number higher than the actual number of colors");
#endif

  if (theWindow->noFont == 0) return;
  if (theWindow->horizontalTextAlignment == GrCenterAlign)
    xOffset = (int) (-grGetTextWidth(theWindow,s)/2);
  if (theWindow->horizontalTextAlignment == GrRightAlign)
    xOffset = (int) (-grGetTextWidth(theWindow,s));
  if (theWindow->verticalTextAlignment == GrCenterAlign)
    yOffset = grGetTextHeight(theWindow)/2;
  if (theWindow->verticalTextAlignment == GrTopAlign)
    yOffset = grGetTextHeight(theWindow);
  XSetForeground(theWindow->dpy,theWindow->gc,
    theWindow->colors[color%theWindow->noColors].pixel);
  XDrawString(theWindow->dpy,theWindow->wd,theWindow->gc,x+xOffset,y+yOffset,s,len);
  if (theWindow->speed == GrSlow) XFlush(theWindow->dpy);
}

void grDrawFillChar(grWindow *theWindow,int x,int y,char c,int fg,int bg) {
  char s[2];
  int xOffset=0,yOffset=0;
  unsigned long oldBackground = theWindow->gc->values.background;
#ifdef DEBUG
  SETFUNC("void grDrawFillChar(grWindow *,int,int,char,int,int)");
  if (fg < 0 OR bg < 0) ERROR("Trying to draw filled character with a negative color number");
  if (fg > theWindow->noColors OR bg > theWindow->noColors) 
    WARNING("Trying to draw character with a"
    " color number higher than the actual number of colors");
#endif

  s[0] = c;s[1] = 0;
  if (theWindow->noFont == 0) return;
  if (theWindow->horizontalTextAlignment == GrCenterAlign)
    xOffset = -grGetTextWidth(theWindow,s)/2;
  if (theWindow->horizontalTextAlignment == GrRightAlign)
    xOffset = -grGetTextWidth(theWindow,s);
  if (theWindow->verticalTextAlignment == GrCenterAlign)
    yOffset = grGetTextHeight(theWindow)/2;
  if (theWindow->verticalTextAlignment == GrTopAlign)
    yOffset = grGetTextHeight(theWindow);
  XSetBackground(theWindow->dpy,theWindow->gc,
    theWindow->colors[bg%theWindow->noColors].pixel);
  XSetForeground(theWindow->dpy,theWindow->gc,
    theWindow->colors[fg%theWindow->noColors].pixel);
  XDrawImageString(theWindow->dpy,theWindow->wd,theWindow->gc,x+xOffset,y+yOffset,s,1);
  if (theWindow->speed == GrSlow) XFlush(theWindow->dpy);
  XSetBackground(theWindow->dpy,theWindow->gc,
    theWindow->colors[oldBackground%theWindow->noColors].pixel);
}

void grDrawFillString(grWindow *theWindow,int x,int y,char *s,int fg,int bg) {
  int xOffset=0,yOffset=0,len=(int) strlen(s);
  unsigned long oldBackground = theWindow->gc->values.background;
#ifdef DEBUG
  SETFUNC("void grDrawFillString(grWindow *,int,int,char *,int,int)");
  if (fg < 0 OR bg < 0) ERROR("Trying to draw filled string with a negative color number");
  if (fg > theWindow->noColors OR bg > theWindow->noColors) 
    WARNING("Trying to draw string with a"
    " color number higher than the actual number of colors");
#endif

  if (theWindow->noFont == 0) return;
  if (theWindow->horizontalTextAlignment == GrCenterAlign)
    xOffset = (int) -grGetTextWidth(theWindow,s)/2;
  if (theWindow->horizontalTextAlignment == GrRightAlign)
    xOffset = (int) -grGetTextWidth(theWindow,s);
  if (theWindow->verticalTextAlignment == GrCenterAlign)
    yOffset = (int) grGetTextHeight(theWindow)/2;
  if (theWindow->verticalTextAlignment == GrTopAlign)
    yOffset = (int) grGetTextHeight(theWindow);
  XSetBackground(theWindow->dpy,theWindow->gc,
    theWindow->colors[bg%theWindow->noColors].pixel);
  XSetForeground(theWindow->dpy,theWindow->gc,
    theWindow->colors[fg%theWindow->noColors].pixel);
  XDrawImageString(theWindow->dpy,theWindow->wd,theWindow->gc,x+xOffset,y+yOffset,s,len);
  if (theWindow->speed == GrSlow) XFlush(theWindow->dpy);
  XSetBackground(theWindow->dpy,theWindow->gc,
    theWindow->colors[oldBackground%theWindow->noColors].pixel);
}
