/*      file     : parallel/TBaseMessage.h
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TBaseMessage.h,v 1.13 2005/03/25 11:06:20 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TBASEMESSAGE_H
#define TBASEMESSAGE_H

// ***INCLUDES************************************************************************ 

#include <TBaseStream.h>
#include <TDLList.h>

#include "ParallelTypes.h"

// ***FORWARDS************************************************************************

class TCommunicationControl;
class TDestination;

// ***DEFINES************************************************************************* 


#define NULL_EVENT_ID            INVALID_ID

#define MSG_FLAG_EVENT           0x00000001
#define MSG_FLAG_AUTO            0x00000002
#define MSG_FLAG_DELETE          0x00000004
#define MSG_FLAG_FLUSHED         0x00000008
#define MSG_FLAG_BASE            0x00000016

#define MSG_STATE_NEW            0
#define MSG_STATE_NEW_RECEIVE    1
#define MSG_STATE_NEW_SEND       2
#define MSG_STATE_RECEIVING      3
#define MSG_STATE_SENDING        4
#define MSG_STATE_RECEIVED       5
#define MSG_STATE_SENT           6

#define MTYPE_EVENT              0

// ***TYPES*************************************************************************** 

typedef long TMessageState;

// ***CLASSES************************************************************************* 

/** Base class to all message classes.
 */

class TBaseMessage : public TBaseStream, public TDLNodePrimitive

{
 public:

    /** Default constructor for an incoming message. 
      Note: this constructor should never be called directly, since the class TCommunication
      handles all incoming messages. */
    TBaseMessage(TCommunicationControl *theControl, TCPNID theSender);

    /** Default constructor for an outgoing message.
      Note: this constructor should never be called directly, since the class TCommunication 
      handles all outgoing messages. */
    TBaseMessage(TCommunicationControl *theControl, TMessageID theID, TCPNID theSender, 
                 TDestination *theDestination,
                 TMessageTag theTag, TMessageType theType, TMessageFlags theFlags);

    virtual ~TBaseMessage(void);
    
    virtual bool
        BeginReceive(void);
    virtual bool
        BeginSend(void);

    /** Temporarily deactivate a message. PVM has problems handling multiple message
      buffers without explicitly switching between them. Deselect should be called whenever
      another message is received or sent while still processing an old one. Use #Select# to
      activate a message again. This is done automatically for event messages before the
      #HandleEvent# method is called for each event. */

    virtual bool
        Deselect(void);
    virtual bool
        EndReceive(void);
    virtual bool
        EndSend(void);
    virtual bool
        Flush(void);
    virtual bool
        EventPending(TEventID &anEventID);
    inline const TDestination *
        GetDestination(void);
    inline TMessageFlags
        GetFlags(void) const;
    inline TMessageID
        GetID(void) const;
    virtual TString
        GetInstanceName(void) const;
    inline TMessageTag
        GetTag(void) const;
    inline TMessageType
        GetType(void) const;
    inline TCPNID
        GetSender(void) const;
    inline TMessageState
        GetState(void) const;
    void
        _Init(void);

    inline bool
    IsFlushed(void) const;

    /** Activate a message which was deactivated through #Deselect#. */

    virtual bool
        Select(void);

/* @Include "TBaseMessage.Headers.h" */
/* #include "TBaseMessage.Headers.h" */

protected:

    virtual TString
        GetClassName(void) const;

private:

    TCommunicationControl *Control;

    TCPNID                 Sender;
    TDestination          *Destination;
    TMessageID             ID;
    TMessageTag            Tag;
    TMessageType           Type;
    TMessageFlags          Flags;

    int                    State;
};


// ***TYPES*************************************************************************** 

typedef TDLList<TBaseMessage> TBaseMessageList;
typedef TDLListEnum<TBaseMessage> TBaseMessageEnum;

// ***INLINE METHODS******************************************************************
#ifndef NO_INLINE
#include "TBaseMessage.i.C"
#endif
// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
