/*      file     : parallel/TCPNInfo.h
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TCPNInfo.h,v 1.6 2005/03/25 11:06:20 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TCPNINFO_H
#define TCPNINFO_H

// ***INCLUDES************************************************************************ 

#include <TString.h>
#include <TDLList.h>
#include <TBaseObject.h>
#include "ParallelTypes.h"

// ***FORWARDS************************************************************************

class TBaseMessage;

// ***DEFINES************************************************************************* 

#define CPNINFO_FLAG_MODIFIED  0x00000001

// ***CLASSES************************************************************************* 

/** Information about one CPN. */

class TCPNInfo : public TBaseObject, public TDLNodePrimitive

{
public:
    
    TCPNInfo(void);
    TCPNInfo(const TCPNInfo *aCPNInfo);
    TCPNInfo(TBaseMessage *aMessage);
    
    void
        operator = (TCPNInfo aCPNInfo);
    bool
        operator == (const TCPNInfo &aCPNInfo) const;
    bool
        Encode(TBaseMessage *aMessage) const;
    bool
        Decode(TBaseMessage *aMessage);
    virtual TString
        GetClassName(void) const;
    virtual TString
        GetInstanceName(void) const;
    
    /** Unique ID of CPN in parallel topology. */
    TCPNID     ID;

    /** Unique ID of host that CPN resides on. */
    THostID    HostID;

    /** Logical name of CPN. */
    TString    Name;

    long       Flags;

    /** Performance of CPN in arbitrary units. */
    double     Performance;

    unsigned long DebugMask;

private:

    void
        _Init(void);
};

// ***TYPES*************************************************************************** 

typedef TDLList<TCPNInfo> TCPNInfoList;
typedef TDLListEnum<TCPNInfo> TCPNInfoEnum;

// ***PROTOTYPES********************************************************************** 

int
CompareCPNInfosByHostAndID(const TCPNInfo **Info1, const TCPNInfo **Info2);

int
CompareCPNInfosByID(const TCPNInfo **Info1, const TCPNInfo **Info2);

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
