/*      file     : TLoadStat.C
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TLoadStat.C,v 1.7 2005/03/25 11:06:49 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include <TBaseStream.h>

#include "TLoadStat.h"

// ***GLOBAL VARIABLES****************************************************************  

TClassInfo *TLoadStat::ClassInfo = 0;

// ***METHODS************************************************************************* 

#ifdef NO_INLINE
#  include "TLoadStat.i.C"
#endif

// =================================================================================== 
#define ParentClass TStatObject
// =================================================================================== 

TLoadStat::TLoadStat(TClassInfo *theClassInfo, TParallelID theParallelID) :

ParentClass(theClassInfo, theParallelID)

{
    Reset();
}

/* =================================================================================== */

TLoadStat::TLoadStat(TParallelID theParallelID) :

ParentClass(ClassInfo, theParallelID)

{
    Reset();
}

/* =================================================================================== */

bool
TLoadStat::Accumulate(const TStatObject *StatObject)

{
    const TLoadStat *Stat = (const TLoadStat *) StatObject;

    IdleTime += Stat->IdleTime;
    ApplicationTime += Stat->ApplicationTime;
    ApplicationLoad += Stat->ApplicationLoad;
    Perf += Stat->Perf;

    if (Stat->MinIdleTime < MinIdleTime)
        MinIdleTime = Stat->MinIdleTime;
    if (Stat->MaxIdleTime > MaxIdleTime)
        MaxIdleTime = Stat->MaxIdleTime;

    if (Stat->MinApplicationTime < MinApplicationTime)
        MinApplicationTime = Stat->MinApplicationTime;
    if (Stat->MaxApplicationTime > MaxApplicationTime)
        MaxApplicationTime = Stat->MaxApplicationTime;

    if (Stat->MinStatTime < MinStatTime)
        MinStatTime = Stat->MinStatTime;
    if (Stat->MaxStatTime > MaxStatTime)
        MaxStatTime = Stat->MaxStatTime;

    if (Stat->MinPerf < Perf)
        MinPerf = Stat->MinPerf;
    if (Stat->MaxPerf > Perf)
        MaxPerf = Stat->MaxPerf;

    if (Stat->MinApplicationLoad < MinApplicationLoad)
        MinApplicationLoad = Stat->MinApplicationLoad;
    if (Stat->MaxApplicationLoad > MaxApplicationLoad)
        MaxApplicationLoad = Stat->MaxApplicationLoad;

    return ParentClass::Accumulate(StatObject);
}

/* =================================================================================== */

bool
TLoadStat::Collect(int theInterval, float theIdleTime, float theMiscTime,
                   float theApplicationTime, float theStatTime,
                   float theApplicationLoad, float thePerf)

{
    Interval = theInterval;

    IdleTime = theIdleTime;
    MinIdleTime = MaxIdleTime = IdleTime;

    MiscTime = theMiscTime;
    MinMiscTime = MaxMiscTime = MiscTime;

    ApplicationTime = theApplicationTime;
    MinApplicationTime = MaxApplicationTime = ApplicationTime;

    StatTime = theStatTime;
    MinStatTime = MaxStatTime = StatTime;

    Perf = thePerf;
    MinPerf = MaxPerf = Perf;

    ApplicationLoad = theApplicationLoad;
    MinApplicationLoad = MaxApplicationLoad = ApplicationLoad;

    return TRUE;
}

/* =================================================================================== */

bool
TLoadStat::Decode(TBaseStream *aStream, TTransferInfo *TransferInfo)

{
    return (ParentClass::Decode(aStream, TransferInfo) AND
            aStream->Decode(Interval, TLOADSTAT_INT) AND
            aStream->Decode(IdleTime, TLOADSTAT_FLOAT));
}

/* =================================================================================== */

bool
TLoadStat::Encode(TBaseStream *aStream, TTransferInfo *TransferInfo) const

{
    return (ParentClass::Encode(aStream, TransferInfo) AND
            aStream->Encode(Interval, TLOADSTAT_INT) AND
            aStream->Encode(IdleTime, TLOADSTAT_FLOAT));
}

/* =================================================================================== */

TString
TLoadStat::GetClassName(void) const

{
    return "TLoadStat";
}

/* =================================================================================== */

void
TLoadStat::PrintScreenStat(FILE *OutputFile)

{
    fprintf(OutputFile, "IdleTime (%.3f, %.3f, %.3f) AppTime (%.3f, %.3f, %.3f) "
            "StatTime (%.3f, %.3f, %.3f) Perf (%.2f, %.2f, %.2f)\n",
            GetMinIdleTime(), GetAverageIdleTime(), GetMaxIdleTime(),
            GetMinApplicationTime(), GetAverageApplicationTime(), GetMaxApplicationTime(),
            GetMinStatTime(), GetAverageStatTime(), GetMaxStatTime(),
            GetMinPerf(), GetAveragePerf(), GetMaxPerf());
}

/* =================================================================================== */

void
TLoadStat::Reset(void)

{
    ApplicationLoad = 0.0;
    ApplicationTime = 0.0;
    StatTime = 0.0;
    MiscTime = 0.0;
    IdleTime = 0.0;
    Perf = 0.0;
}

// =================================================================================== 

bool
TLoadStat::Write(const TTimeStep &BaseTimeStep, TBaseStream *Stream)

{
    Stream->Write("%ld %ld %f %f %f %f %f %f %f %f %f %f\n",
                  (long) GetTimeStep(),
                  (long) BaseTimeStep + (long) GetTimeStep(),
                  (double) ((long) GetTimeStep()) / 3600.0,
                  GetMinIdleTime(), GetAverageIdleTime(), GetMaxIdleTime(),
                  GetMinApplicationTime(), GetAverageApplicationTime(), GetMaxApplicationTime(),
                  GetMinPerf(), GetAveragePerf(), GetMaxPerf(),
                  GetMinStatTime(), GetAverageStatTime(), GetMaxStatTime());
    return TRUE;    
}

// =================================================================================== 

bool
TLoadStat::WriteStatDescription(TBaseStream *Stream)

{
    int Col = 0;
    Stream->Write("# column description:\n");
    Stream->Write("# %2d : simulation timestep\n", ++Col);
    Stream->Write("# %2d : application timestep\n", ++Col);
    Stream->Write("# %2d : application time\n", ++Col);
    Stream->Write("# %2d : minimum idle time [s/timestep]\n", ++Col);
    Stream->Write("# %2d : average idle time [s/timestep]\n", ++Col);
    Stream->Write("# %2d : maximum idle time [s/timestep]\n", ++Col);
    Stream->Write("# %2d : minimum application time [s/timestep]\n", ++Col);
    Stream->Write("# %2d : average application time [s/timestep]\n", ++Col);
    Stream->Write("# %2d : maximum application time [s/timestep]\n", ++Col);
    Stream->Write("# %2d : minimum performance\n", ++Col);
    Stream->Write("# %2d : average performance\n", ++Col);
    Stream->Write("# %2d : maximum performance\n", ++Col);
    Stream->Write("# %2d : minimum stat time [s/timestep]\n", ++Col);
    Stream->Write("# %2d : average stat time [s/timestep]\n", ++Col);
    Stream->Write("# %2d : maximum stat time [s/timestep]\n", ++Col);
    return TRUE;    
}

// =================================================================================== 
#undef ParentClass
