/*      file     : parbalance/TLoadStat.h
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TLoadStat.h,v 1.7 2005/03/25 11:06:49 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TLoadStat_H
#define TLoadStat_H

// ***INCLUDES************************************************************************ 

#include <stdio.h>

#include <TDLList.h>
#include <TStatObject.h>

// ***CLASSES************************************************************************* 

class TLoadStat : public TStatObject

{
public:

    TLoadStat(TClassInfo *theClassInfo, TParallelID theParallelID = INVALID);
    TLoadStat(TParallelID theParallelID = INVALID);

    bool
    Collect(int theInterval, float theIdleTime, float theMiscTime,
            float theApplicationTime, float theStatTime,
            float theApplicationLoad, float thePerf);

    inline double
    GetAverageApplicationLoad(void) const;

    inline double
    GetAverageApplicationTime(void) const;

    inline double
    GetAverageStatTime(void) const;

    inline double
    GetAverageEffectiveTime(void) const;

    inline double
    GetAverageIdleTime(void) const;

    inline double
    GetAverageMiscTime(void) const;

    inline double
    GetAverageSumTime(void) const;

    inline double
    GetAveragePerf(void) const;

    inline double
    GetMinApplicationTime(void) const;

    inline double
    GetMaxApplicationTime(void) const;

    inline double
    GetMinStatTime(void) const;

    inline double
    GetMaxStatTime(void) const;

    inline double
    GetMinIdleTime(void) const;

    inline double
    GetMaxIdleTime(void) const;

    inline double
    GetMinMiscTime(void) const;

    inline double
    GetMaxMiscTime(void) const;

    inline double
    GetMinPerf(void) const;

    inline double
    GetMaxPerf(void) const;

    void
    PrintScreenStat(FILE *OutputFile);
    static TClassInfo *ClassInfo;

    bool
    Write(const TTimeStep &BaseTimeStep, TBaseStream *Stream);

    static bool
    WriteStatDescription(TBaseStream *Stream);

protected:

    virtual bool
    Decode(TBaseStream *aStream, TTransferInfo *TransferInfo);

    virtual bool
    Encode(TBaseStream *aStream, TTransferInfo *TransferInfo) const;

    virtual TString
    GetClassName(void) const;

private:

    virtual bool
    Accumulate(const TStatObject *StatObject);

    virtual void
    Reset(void);

private:

    //  >>> DO NOT SEPERATE <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
#   define TLOADSTAT_INT 1
    int   Interval;
#   define TLOADSTAT_FLOAT 18
    float IdleTime;
    float MinIdleTime;
    float MaxIdleTime;
    float MiscTime;
    float MinMiscTime;
    float MaxMiscTime;
    float Perf;
    float MinPerf;
    float MaxPerf;
    float ApplicationLoad;
    float MinApplicationLoad;
    float MaxApplicationLoad;
    float ApplicationTime;
    float MinApplicationTime;
    float MaxApplicationTime;
    float StatTime;
    float MinStatTime;
    float MaxStatTime;
    //  >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
};

// ***TYPES***************************************************************************

typedef TDLList < TLoadStat > TLoadStatList;
typedef TDLListEnum < TLoadStat > TLoadStatEnum;

// ***INLINE METHODS******************************************************************

#ifndef NO_INLINE
#  include "TLoadStat.i.C"
#endif

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
