/*      file     : partools/TContainerObject.h
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TContainerObject.h,v 1.15 2005/03/25 11:08:14 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TCONTAINEROBJECT_H
#define TCONTAINEROBJECT_H

// ***INCLUDES************************************************************************ 

#include <TDLList.h>

#include <TParallelListObject.h>

#include <ParToolsTypes.h>

// ***FORWARDS************************************************************************

class TBaseStream;
class TTransferInfo;

// ***CLASSES************************************************************************* 

struct TContainerReference 

{
    TClassID    ClassID;
    TParallelID ParallelID;
};

// ===================================================================================

/**
  */

class TContainerObject : public TParallelListObject

{
public:

    TContainerObject(TClassInfo *ClassInfo, TParallelID theParallelID);
    virtual ~TContainerObject(void);

    inline void
    AddInstance(TInfoMask aLocalMask = 0);

    virtual bool
    Decode(TBaseStream *Stream, TTransferInfo *TransferInfo);

    virtual bool
    DetermineCPNID(void);

    void
    DisposeNonLocalInfos(void);

    virtual bool
    DisposeNonLocalPortion(TCPNID CPNID);

    virtual bool
    Encode(TBaseStream *Stream, TTransferInfo *TransferInfo) const;

    virtual double
    GetActiveFraction(TCPNID CPNID) const;

    virtual bool
    GetBoundaries(TTimeStep &TimeStep, TDLListPrimitive *Boundaries);

    virtual TCPNID
    GetCPNID(void) const;

    virtual bool
    GetDependantContainers(int &NrOfRefs, TContainerReference *);

    virtual TParallelObject *
    GetInfo(TInfoID theInfoID, bool Create = FALSE, bool Load = FALSE) const;

    virtual TString
    GetInstanceName(void) const;

    virtual double
    GetLoad(TCPNID LocalCPNID, TInfoID AdjacenceInfoID, TInfoID LocationInfoID);

    inline TInfoMask
    GetLoadableMask(void) const;
    
    inline TInfoMask
    GetLocalMask(void) const;
    
    inline int
    GetNrOfInstances(void) const;

    virtual bool
    GetRequiredContainers(int &NrOfRefs, TContainerReference *);

    inline TInfoMask
    GetRequiredMask(void) const;

    virtual bool
    IsDistributable(void) const;

    virtual bool
    IsLocal(TCPNID CPNID) const;

    void
    RemoveInstance(void);

    inline void
    SetCPNID(TCPNID theCPNID);

    inline void
    SetInfo(TInfoID theInfoID, const TParallelObject *theInfo, bool DeletePrevious = TRUE);

    inline void
    SetLoadableMask(TInfoMask theMask);

    inline void
    SetRequiredMask(TInfoMask theMask);

    bool
    StartTimer(TInfoID InfoID);

    bool
    StopTimer(TInfoID InfoID, int Interval = 1);

private:

    TCPNID            CPNID;
    int               NrOfInstances;
    TInfoMask         LoadableMask;
    TInfoMask         RequiredMask;
    TInfoMask         LocalMask;
    TParallelObject **Infos;
};

// ***TYPES*************************************************************************** 

typedef TDLList< TContainerObject > TContainerObjectList;
typedef TDLListEnum< TContainerObject > TContainerObjectEnum;

// ***INLINE METHODS******************************************************************

#ifndef NO_INLINE
#  include "TContainerObject.i.C"
#endif

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
