/*      file     : partools/TDuration.h
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TDuration.h,v 1.5 2005/03/25 11:08:14 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TDURATION_H
#define TDURATION_H

// ***INCLUDES************************************************************************ 

#include <TTimeStep.h>
#include <TBaseObject.h>

// ***FORWARDS************************************************************************

class TBaseStream;
class TTimeStep;

// ***CLASSES************************************************************************* 

/** Defines a time range between two time-steps. It can handle special cases like
  \begin{itemize}
  \item active until a given time-step
  \item active starting at a given time-step
  \item always active
  \item activate and deactivate
  \end{itemize}
  */

class TDuration : public TBaseObject

{
public:

    TDuration(void);
    TDuration(const TTimeStep &theStart, const TTimeStep &theEnd);
    TDuration(const TTimeStep &theStart, long Duration);

    inline bool
        Active(TTimeStep aTimeStep) const;
    void
        Activate(void);
    void
        Deactivate(void);
    bool
        Decode(TBaseStream *aStream);
    bool
        Encode(TBaseStream *aStream) const;

    inline TTimeStep &
    GetStart(void);

    inline const TTimeStep &
    GetStart(void) const;

    inline TTimeStep &
    GetEnd(void);

    inline const TTimeStep &
    GetEnd(void) const;

private:

    TTimeStep Start;
    TTimeStep End;
};

// ***INLINE METHODS******************************************************************

#ifndef NO_INLINE
#  include "TDuration.i.C"
#endif

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
