/*      file     : partools/TNetwork.C
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TNetwork.C,v 1.3 2005/03/25 11:08:15 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include <TBaseObject.h>

#include <TParallel.h>
#include <TTopology.h>
#include <TCommunicationLink.h>
#include <TExtendedCPNInfo.h>
#include <TCPNInfo.h>

#include "TNetwork.h"

// ***DEFINES************************************************************************* 
// ***CONSTANTS***********************************************************************
// ***TYPES*************************************************************************** 
// ***GLOBAL VARIABLES****************************************************************  

TNetwork *Network = 0;

// ***LOCAL VARIABLES***************************************************************** 
// ***FUNCTIONS*********************************************************************** 
// ***METHODS************************************************************************* 

#ifdef NO_INLINE
#endif

// =================================================================================== 

// =================================================================================== 
#define ParentClass TBaseObject
// =================================================================================== 

TNetwork::TNetwork(void)

{
    if (Network != 0)
        Warning("another instance of TNetwork active");
    Network = this;
}

// =================================================================================== 

TNetwork::~TNetwork(void)

{
    Network = 0;
}

// =================================================================================== 

TCommunicationLink *
TNetwork::GetCommunicationLink(TParallelID LinkID, bool Create)

{
    TCommunicationLink *Link;

    if ((Link = (TCommunicationLink *) 
         TCommunicationLink::ClassInfo->GetObjectPtr(LinkID, FALSE)))
        return Link;
    if (!Create)
    {
        Warning("GetCommunicationLink: could not find communication link %ld", LinkID);
        return 0;
    }
    return new TCommunicationLink(LinkID);
}

// =================================================================================== 

TCommunicationLink *
TNetwork::GetCommunicationLink(TCPNID FromCPNID, TCPNID ToCPNID, bool Create)

{
    TCommunicationLink *Link;
    TParallelID         LinkID;

    LinkID = GetCommunicationLinkID(FromCPNID, ToCPNID);
    if ((Link = (TCommunicationLink *) 
         TCommunicationLink::ClassInfo->GetObjectPtr(LinkID, FALSE)))
        return Link;
    if (!Create)
    {
        Warning("GetCommunicationLink: could not find communication link %ld", LinkID);
        return 0;
    }
    return new TCommunicationLink(FromCPNID, ToCPNID);
}

// =================================================================================== 

long
TNetwork::GetCommunicationLinkID(TCPNID ID1, TCPNID ID2)

{
    TTopology *Topology;
    TCPNInfo  *Info1;
    TCPNInfo  *Info2;
    long       ParallelID;

    if ((Topology = Parallel->GetPhysicalTopology()) == 0)
    {
        Error("GetLinkID: cannot retrieve topology");
        return INVALID;
    }
    if ((Info1 = Parallel->GetCPNInfoByID(ID1)) == 0)
    {
        Error("GetLinkID: cannot retrieve CPNInfo %lX", ID1);
        return INVALID;
    }
    if ((Info2 = Parallel->GetCPNInfoByID(ID2)) == 0)
    {
        Error("GetLinkID: cannot retrieve CPNInfo %lX", ID2);
        return INVALID;
    }
    ParallelID = (Topology->GetRank(Info1) << 16) | Topology->GetRank(Info2);
    return ParallelID;
}

// =================================================================================== 

TExtendedCPNInfo *
TNetwork::GetExtendedCPNInfo(TCPNID SearchCPNID, bool Create)

{
    TCPNInfo         *CPNInfo;
    TExtendedCPNInfo *ExtendedCPNInfo;

    if ((ExtendedCPNInfo = 
         (TExtendedCPNInfo *) TExtendedCPNInfo::ClassInfo->GetObjectPtr(SearchCPNID, FALSE)))
        return ExtendedCPNInfo;
    if (!Create)
    {
        Warning("GetExtendedCPNInfo: could not find ExtendedInfo for CPNID %lX", SearchCPNID);
        return 0;
    }
    if ((CPNInfo = Parallel->GetCPNInfoByID(SearchCPNID)) == 0)
    {
        Error("GetExtendedCPNInfo: could not find CPNInfo for CPNID %ld", SearchCPNID);
        return 0;
    }
    ExtendedCPNInfo = new TExtendedCPNInfo(CPNInfo);
    return ExtendedCPNInfo;
}


// =================================================================================== 

TCommunicationLinkTable *
TNetwork::GetCommunicationLinkTable(void)

{
    return (TCommunicationLinkTable *) TCommunicationLink::ClassInfo->GetTable();
}

// =================================================================================== 

bool
TNetwork::SplitCommunicationLinkID(long ParallelID, TCPNID &ID1, TCPNID &ID2)

{
    TTopology *Topology;
    int        Rank1;
    int        Rank2;
    const TCPNInfo  *Info1;
    const TCPNInfo  *Info2;

    if ((Topology = Parallel->GetPhysicalTopology()) == 0)
    {
        Error("SplitCommunicationLinkID: cannot retrieve topology");
        return FALSE;
    }
    Rank1 = (int) (ParallelID >> 16);
    Rank2 = (int) (ParallelID & 0xFFFF);
    if ((Info1 = Topology->GetCPNInfoByRank(Rank1)) == 0)
    {
        Error("SplitCommunicationLinkID: cannot retrieve rank %d", Rank1);
        return FALSE;
    }
    ID1 = Info1->ID;
    if ((Info2 = Topology->GetCPNInfoByRank(Rank2)) == 0)
    {
        Error("SplitCommunicationLinkID: cannot retrieve rank %d", Rank2);
        return FALSE;
    }
    ID2 = Info2->ID;
    return TRUE;
}

// =================================================================================== 
#undef ParentClass
