/*      file     : partools/TNode.h
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TNode.h,v 1.9 2005/03/25 11:08:15 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TNODE_H
#define TNODE_H

// ***INCLUDES************************************************************************

#include <TDLList.h>
#include <TLocationInfo.h>

#include "TContainerObject.h"
 
// ***FORWARDS************************************************************************

class TClassInfo;
class TIncidenceInfo;

// ***CLASSES************************************************************************* 

/** Base class to the simulation node class.
  Associated Infos:
  \begin{itemize}
  \item geographic data #TLocationInfo#
  \item algorithmic data 
  \item intersection data #Intersection#
  \item phasing data #TIntersectionControl#
  \end{itemize}
  */

class TNode : public TContainerObject

{
public:

    TNode(TClassInfo *theClassInfo, TParallelID theParallelID, 
          TInfoID theDefaultIncidenceInfoID);

    inline TIncidenceInfo *
    GetIncidenceInfo(TInfoID IncidenceInfoID = INVALID);

    virtual double
    GetLoad(TCPNID LocalCPNID, TInfoID AdjacenceInfoID, TInfoID LocationInfoID);

    static TNode *
    GetPtr(TParallelID ID);

    static TNode *
    LoadInstance(TParallelID NodeID, TInfoMask LocalInfoMask = 0);

    inline TLocation &
    Loc(TInfoID LocationInfoID);

    static TClassInfo *ClassInfo;

private:

    virtual bool
    Decode(TBaseStream *Stream, TTransferInfo *TransferInfo);

    virtual bool
    Encode(TBaseStream *Stream, TTransferInfo *TransferInfo) const;

    virtual TString
    GetClassName(void) const;

    virtual bool
    GetDependantContainers(int &NrOfRefs, TContainerReference *);

    TInfoID DefaultIncidenceInfoID;
};

// ***TYPES*************************************************************************** 

typedef TDLList< TNode > TNodeList;
typedef TDLListEnum< TNode > TNodeEnum;

// ***INLINE METHODS******************************************************************

#ifndef NO_INLINE
#  include "TNode.i.C"
#endif

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
