/*      file     : partools/TTimeStep.C
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TTimeStep.C,v 1.4 2005/03/25 11:08:15 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************

#include <stdio.h>

#include <TString.h>
#include <TBaseStream.h>

#include "TTimeStep.h"

// ***LOCAL VARIABLES*****************************************************************

short int TTimeStep::NrOfSubTimeSteps = 1;

// ***METHODS************************************************************************* 

#ifdef NO_INLINE
#  include <TTimeStep.i.C>
#endif

// =================================================================================== 
#define ParentClass TBaseObject
// =================================================================================== 

TTimeStep::TTimeStep(long theTimeStep, short int theSubTimeStep)

{
    TimeStep = theTimeStep;
    SubTimeStep = theSubTimeStep;
}
                     
// ===================================================================================

long
TTimeStep::operator ++ (int)

{
    SubTimeStep++;
    if (SubTimeStep == NrOfSubTimeSteps)
    {
        SubTimeStep = 0;
        TimeStep++;
    }
    return TimeStep;
}

// ===================================================================================

bool
TTimeStep::Decode(TBaseStream *aStream)

{
    long Temp;

    if (!aStream->Decode(Temp))
        return FALSE;
    TimeStep = Temp / NrOfSubTimeSteps;
    SubTimeStep = (short int) (Temp % (long) NrOfSubTimeSteps);
    return TRUE;
}

// ===================================================================================

bool
TTimeStep::Encode(TBaseStream *aStream) const

{
    long Temp;

    Temp = GetLinear();
    return aStream->Encode(Temp);
}

// ===================================================================================

TString
TTimeStep::GetClassName(void) const

{
    return "TTimeStep";
}

// ===================================================================================

TString
TTimeStep::GetInstanceName(void) const

{
    char Line[128];

    sprintf(Line, "%ld:%d", TimeStep, SubTimeStep);
    return Line;
}

// ===================================================================================

void
TTimeStep::SetNrOfSubTimeSteps(int theNrOfSubTimeSteps)

{
    NrOfSubTimeSteps = theNrOfSubTimeSteps;
}

// =================================================================================== 
#undef ParentClass


